@@ -22,7 +22,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.81.2.25 2005/07/11 19:29:56 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.81.2.77 2009/03/19 04:54:50 shadow Exp $");
 
 #include "afs/sysincludes.h"
 #include "afsincludes.h"
@@ -45,11 +45,17 @@ RCSID
 #define pageoff(pp) pp->offset
 #endif
 
+#ifndef MAX_ERRNO
+#define MAX_ERRNO 1000L
+#endif
+
 #if defined(AFS_LINUX26_ENV)
 #define UnlockPage(pp) unlock_page(pp)
+extern struct backing_dev_info afs_backing_dev_info;
 #endif
 
 extern struct vcache *afs_globalVp;
+extern int afs_notify_change(struct dentry *dp, struct iattr *iattrp);
 
 
 static ssize_t
@@ -73,50 +79,14 @@ afs_linux_read(struct file *fp, char *buf, size_t count, loff_t * offp)
     if (code)
 	code = -code;
     else {
-#ifdef AFS_64BIT_CLIENT
-	if (*offp + count > afs_vmMappingEnd) {
-	    uio_t tuio;
-	    struct iovec iov;
-	    afs_int32 xfered = 0;
-
-	    if (*offp < afs_vmMappingEnd) {
-		/* special case of a buffer crossing the VM mapping end */
-		afs_int32 tcount = afs_vmMappingEnd - *offp;
-		count -= tcount;
-		osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
-		AFS_GUNLOCK();
-		code = generic_file_read(fp, buf, tcount, offp);
-		AFS_GLOCK();
-		if (code != tcount) {
-		    goto done;
-		}
-		xfered = tcount;
-	    }
-	    setup_uio(&tuio, &iov, buf + xfered, (afs_offs_t) * offp, count,
-		      UIO_READ, AFS_UIOSYS);
-	    code = afs_read(vcp, &tuio, credp, 0, 0, 0);
-	    xfered += count - tuio.uio_resid;
-	    if (code != 0) {
-		afs_Trace4(afs_iclSetp, CM_TRACE_READOP, ICL_TYPE_POINTER,
-			   vcp, ICL_TYPE_OFFSET, offp, ICL_TYPE_INT32, -1,
-			   ICL_TYPE_INT32, code);
-		code = xfered;
-		*offp += count - tuio.uio_resid;
-	    } else {
-		code = xfered;
-		*offp += count;
-	    }
-	  done:
-		;
-	} else {
-#endif /* AFS_64BIT_CLIENT */
 	    osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
 	    AFS_GUNLOCK();
+#ifdef DO_SYNC_READ
+	    code = do_sync_read(fp, buf, count, offp);
+#else
 	    code = generic_file_read(fp, buf, count, offp);
+#endif
 	    AFS_GLOCK();
-#ifdef AFS_64BIT_CLIENT
-	}
-#endif /* AFS_64BIT_CLIENT */
     }
 
     afs_Trace4(afs_iclSetp, CM_TRACE_READOP, ICL_TYPE_POINTER, vcp,
@@ -137,11 +107,9 @@ static ssize_t
 afs_linux_write(struct file *fp, const char *buf, size_t count, loff_t * offp)
 {
     ssize_t code = 0;
-    int code2 = 0;
     struct vcache *vcp = VTOAFS(fp->f_dentry->d_inode);
     struct vrequest treq;
     cred_t *credp = crref();
-    afs_offs_t toffs;
 
     AFS_GLOCK();
 
@@ -161,78 +129,17 @@ afs_linux_write(struct file *fp, const char *buf, size_t count, loff_t * offp)
     if (code)
 	code = -code;
     else {
-#ifdef AFS_64BIT_CLIENT
-	toffs = *offp;
-	if (fp->f_flags & O_APPEND)
-	    toffs += vcp->m.Length;
-	if (toffs + count > afs_vmMappingEnd) {
-	    uio_t tuio;
-	    struct iovec iov;
-	    afs_size_t oldOffset = *offp;
-	    afs_int32 xfered = 0;
-
-	    if (toffs < afs_vmMappingEnd) {
-		/* special case of a buffer crossing the VM mapping end */
-		afs_int32 tcount = afs_vmMappingEnd - *offp;
-		count -= tcount;
-		AFS_GUNLOCK();
-		code = generic_file_write(fp, buf, tcount, offp);
-		AFS_GLOCK();
-		if (code != tcount) {
-		    goto done;
-		}
-		xfered = tcount;
-		toffs += tcount;
-	    }
-	    setup_uio(&tuio, &iov, buf + xfered, (afs_offs_t) toffs, count,
-		      UIO_WRITE, AFS_UIOSYS);
-	    code = afs_write(vcp, &tuio, fp->f_flags, credp, 0);
-	    xfered += count - tuio.uio_resid;
-	    if (code != 0) {
-		code = xfered;
-		*offp += count - tuio.uio_resid;
-	    } else {
-		/* Purge dirty chunks of file if there are too many dirty chunks.
-		 * Inside the write loop, we only do this at a chunk boundary.
-		 * Clean up partial chunk if necessary at end of loop.
-		 */
-		if (AFS_CHUNKBASE(tuio.afsio_offset) !=
-		    AFS_CHUNKBASE(oldOffset)) {
-		    ObtainWriteLock(&vcp->lock, 402);
-		    code = afs_DoPartialWrite(vcp, &treq);
-		    vcp->states |= CDirty;
-		    ReleaseWriteLock(&vcp->lock);
-		}
-		code = xfered;
-		*offp += count;
-		toffs += count;
-		ObtainWriteLock(&vcp->lock, 400);
-		vcp->m.Date = osi_Time();	/* Set file date (for ranlib) */
-		/* extend file */
-		if (!(fp->f_flags & O_APPEND) && toffs > vcp->m.Length) {
-		    vcp->m.Length = toffs;
-		}
-		ReleaseWriteLock(&vcp->lock);
-	    }
-	  done:
-		;
-	} else {
-#endif /* AFS_64BIT_CLIENT */
 	    AFS_GUNLOCK();
+#ifdef DO_SYNC_READ
+	    code = do_sync_write(fp, buf, count, offp);
+#else
 	    code = generic_file_write(fp, buf, count, offp);
+#endif
 	    AFS_GLOCK();
-#ifdef AFS_64BIT_CLIENT
-	}
-#endif /* AFS_64BIT_CLIENT */
     }
 
     ObtainWriteLock(&vcp->lock, 530);
-    vcp->m.Date = osi_Time();	/* set modification time */
     afs_FakeClose(vcp, credp);
-    if (code >= 0)
-	code2 = afs_DoPartialWrite(vcp, &treq);
-    if (code2 && code >= 0)
-	code = (ssize_t) - code2;
     ReleaseWriteLock(&vcp->lock);
 
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITEOP, ICL_TYPE_POINTER, vcp,
@@ -424,71 +331,6 @@ static long afs_unlocked_xioctl(struct file *fp, unsigned int com,
 }
 #endif
 
-/* We need to detect unmap's after close. To do that, we need our own
- * vm_operations_struct's. And we need to set them up for both the
- * private and shared mappings. The fun part is that these are all static
- * so we'll have to initialize on the fly!
- */
-static struct vm_operations_struct afs_private_mmap_ops;
-static int afs_private_mmap_ops_inited = 0;
-static struct vm_operations_struct afs_shared_mmap_ops;
-static int afs_shared_mmap_ops_inited = 0;
-
-static void
-afs_linux_vma_close(struct vm_area_struct *vmap)
-{
-    struct vcache *vcp;
-    cred_t *credp;
-    int need_unlock = 0;
-
-    if (!vmap->vm_file)
-	return;
-
-    vcp = VTOAFS(FILE_INODE(vmap->vm_file));
-    if (!vcp)
-	return;
-
-    AFS_GLOCK();
-    afs_Trace4(afs_iclSetp, CM_TRACE_VM_CLOSE, ICL_TYPE_POINTER, vcp,
-	       ICL_TYPE_INT32, vcp->mapcnt, ICL_TYPE_INT32, vcp->opens,
-	       ICL_TYPE_INT32, vcp->execsOrWriters);
-    if ((&vcp->lock)->excl_locked == 0 || (&vcp->lock)->pid_writer == MyPidxx) {
-	ObtainWriteLock(&vcp->lock, 532);
-	need_unlock = 1;
-    } else
-	printk("AFS_VMA_CLOSE(%d): Skipping Already locked vcp=%p vmap=%p\n",
-	       MyPidxx, &vcp, &vmap);
-    if (vcp->mapcnt) {
-	vcp->mapcnt--;
-	if (need_unlock)
-	    ReleaseWriteLock(&vcp->lock);
-	if (!vcp->mapcnt) {
-	    if (need_unlock && vcp->execsOrWriters < 2) {
-		credp = crref();
-		(void)afs_close(vcp, vmap->vm_file->f_flags, credp);
-		/* only decrement the execsOrWriters flag if this is not a
-		 * writable file. */
-		if (!(vcp->states & CRO) )
-		    if (! (vmap->vm_file->f_flags & (FWRITE | FTRUNC)))
-			vcp->execsOrWriters--;
-		vcp->states &= ~CMAPPED;
-		crfree(credp);
-	    } else if ((vmap->vm_file->f_flags & (FWRITE | FTRUNC)))
-		vcp->execsOrWriters--;
-	    /* If we did not have the lock */
-	    if (!need_unlock) {
-		vcp->mapcnt++;
-		if (!vcp->execsOrWriters)
-		    vcp->execsOrWriters = 1;
-	    }
-	}
-    } else {
-	if (need_unlock)
-	    ReleaseWriteLock(&vcp->lock);
-    }
-
-    AFS_GUNLOCK();
-}
 
 static int
 afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
@@ -512,59 +354,29 @@ afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
 
     /* get a validated vcache entry */
     code = afs_InitReq(&treq, credp);
-    if (!code)
-	code = afs_VerifyVCache(vcp, &treq);
-
-    if (!code && (vcp->states & CRO) && 
-	(vmap->vm_file->f_flags & (FWRITE | FTRUNC)))
-	code = EACCES;
-
     if (code)
-	code = -code;
-    else {
-	osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
+	goto out_err;
 
-	AFS_GUNLOCK();
-	code = generic_file_mmap(fp, vmap);
-	AFS_GLOCK();
-    }
+    code = afs_VerifyVCache(vcp, &treq);
+    if (code)
+	goto out_err;
 
-    if (code == 0) {
-	ObtainWriteLock(&vcp->lock, 531);
-	/* Set out vma ops so we catch the close. The following test should be
-	 * the same as used in generic_file_mmap.
-	 */
-	if ((vmap->vm_flags & VM_SHARED) && (vmap->vm_flags & VM_MAYWRITE)) {
-	    if (!afs_shared_mmap_ops_inited) {
-		afs_shared_mmap_ops_inited = 1;
-		afs_shared_mmap_ops = *vmap->vm_ops;
-		afs_shared_mmap_ops.close = afs_linux_vma_close;
-	    }
-	    vmap->vm_ops = &afs_shared_mmap_ops;
-	} else {
-	    if (!afs_private_mmap_ops_inited) {
-		afs_private_mmap_ops_inited = 1;
-		afs_private_mmap_ops = *vmap->vm_ops;
-		afs_private_mmap_ops.close = afs_linux_vma_close;
-	    }
-	    vmap->vm_ops = &afs_private_mmap_ops;
-	}
+    osi_FlushPages(vcp, credp);	/* ensure stale pages are gone */
 
+    AFS_GUNLOCK();
+    code = generic_file_mmap(fp, vmap);
+    AFS_GLOCK();
+    if (!code)
+	vcp->states |= CMAPPED;
 
-	/* Add an open reference on the first mapping. */
-	if (vcp->mapcnt == 0) {
-	    if (!(vcp->states & CRO))
-		vcp->execsOrWriters++;
-	    vcp->opens++;
-	    vcp->states |= CMAPPED;
-	}
-	ReleaseWriteLock(&vcp->lock);
-	vcp->mapcnt++;
-    }
-
+out:
     AFS_GUNLOCK();
     crfree(credp);
     return code;
+
+out_err:
+    code = -code;
+    goto out;
 }
 
 static int
@@ -664,6 +476,25 @@ afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
     code = afs_lockctl(vcp, &flock, cmd, credp);
     AFS_GUNLOCK();
 
+#ifdef AFS_LINUX24_ENV
+    if ((code == 0 || flp->fl_type == F_UNLCK) && 
+        (cmd == F_SETLK || cmd == F_SETLKW)) {
+#ifdef POSIX_LOCK_FILE_WAIT_ARG
+	code = posix_lock_file(fp, flp, 0);
+#else
+	flp->fl_flags &=~ FL_SLEEP;
+	code = posix_lock_file(fp, flp);
+#endif 
+	if (code && flp->fl_type != F_UNLCK) {
+	    struct AFS_FLOCK flock2;
+	    flock2 = flock;
+	    flock2.l_type = F_UNLCK;
+	    AFS_GLOCK();
+	    afs_lockctl(vcp, &flock2, F_SETLK, credp);
+	    AFS_GUNLOCK();
+	}
+    }
+#endif
     /* Convert flock back to Linux's file_lock */
     flp->fl_type = flock.l_type;
     flp->fl_pid = flock.l_pid;
@@ -675,27 +506,90 @@ afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
 
 }
 
+#ifdef STRUCT_FILE_OPERATIONS_HAS_FLOCK
+static int
+afs_linux_flock(struct file *fp, int cmd, struct file_lock *flp) {
+    int code = 0;
+    struct vcache *vcp = VTOAFS(FILE_INODE(fp));
+    cred_t *credp = crref();
+    struct AFS_FLOCK flock;
+    /* Convert to a lock format afs_lockctl understands. */
+    memset((char *)&flock, 0, sizeof(flock));
+    flock.l_type = flp->fl_type;
+    flock.l_pid = flp->fl_pid;
+    flock.l_whence = 0;
+    flock.l_start = 0;
+    flock.l_len = OFFSET_MAX;
+
+    /* Safe because there are no large files, yet */
+#if defined(F_GETLK64) && (F_GETLK != F_GETLK64)
+    if (cmd == F_GETLK64)
+	cmd = F_GETLK;
+    else if (cmd == F_SETLK64)
+	cmd = F_SETLK;
+    else if (cmd == F_SETLKW64)
+	cmd = F_SETLKW;
+#endif /* F_GETLK64 && F_GETLK != F_GETLK64 */
+
+    AFS_GLOCK();
+    code = afs_lockctl(vcp, &flock, cmd, credp);
+    AFS_GUNLOCK();
+
+    if ((code == 0 || flp->fl_type == F_UNLCK) && 
+        (cmd == F_SETLK || cmd == F_SETLKW)) {
+	flp->fl_flags &=~ FL_SLEEP;
+	code = flock_lock_file_wait(fp, flp);
+	if (code && flp->fl_type != F_UNLCK) {
+	    struct AFS_FLOCK flock2;
+	    flock2 = flock;
+	    flock2.l_type = F_UNLCK;
+	    AFS_GLOCK();
+	    afs_lockctl(vcp, &flock2, F_SETLK, credp);
+	    AFS_GUNLOCK();
+	}
+    }
+    /* Convert flock back to Linux's file_lock */
+    flp->fl_type = flock.l_type;
+    flp->fl_pid = flock.l_pid;
+
+    crfree(credp);
+    return -code;
+}
+#endif
+
 /* afs_linux_flush
  * essentially the same as afs_fsync() but we need to get the return
  * code for the sys_close() here, not afs_linux_release(), so call
  * afs_StoreAllSegments() with AFS_LASTSTORE
  */
 static int
+#if defined(FOP_FLUSH_TAKES_FL_OWNER_T)
+afs_linux_flush(struct file *fp, fl_owner_t id)
+#else
 afs_linux_flush(struct file *fp)
+#endif
 {
     struct vrequest treq;
-    struct vcache *vcp = VTOAFS(FILE_INODE(fp));
-    cred_t *credp = crref();
+    struct vcache *vcp;
+    cred_t *credp;
     int code;
 
     AFS_GLOCK();
 
+    if ((fp->f_flags & O_ACCMODE) == O_RDONLY) { /* readers dont flush */
+	AFS_GUNLOCK();
+	return 0;
+    }
+
+    credp = crref();
+    vcp = VTOAFS(FILE_INODE(fp));
+
     code = afs_InitReq(&treq, credp);
     if (code)
 	goto out;
 
     ObtainSharedLock(&vcp->lock, 535);
-    if (vcp->execsOrWriters > 0) {
+    if ((vcp->execsOrWriters > 0) && (file_count(fp) == 1)) {
 	UpgradeSToWLock(&vcp->lock, 536);
 	code = afs_StoreAllSegments(vcp, &treq, AFS_SYNC | AFS_LASTSTORE);
 	ConvertWToSLock(&vcp->lock);
@@ -745,6 +639,10 @@ struct file_operations afs_dir_fops = {
 struct file_operations afs_file_fops = {
   .read =	afs_linux_read,
   .write =	afs_linux_write,
+#ifdef GENERIC_FILE_AIO_READ
+  .aio_read =	generic_file_aio_read,
+  .aio_write =	generic_file_aio_write,
+#endif
 #ifdef HAVE_UNLOCKED_IOCTL
   .unlocked_ioctl = afs_unlocked_xioctl,
 #else
@@ -756,12 +654,19 @@ struct file_operations afs_file_fops = {
   .mmap =	afs_linux_mmap,
   .open =	afs_linux_open,
   .flush =	afs_linux_flush,
-#ifdef AFS_LINUX26_ENV
+#if defined(AFS_LINUX26_ENV) && defined(STRUCT_FILE_OPERATIONS_HAS_SENDFILE)
   .sendfile =   generic_file_sendfile,
+#endif
+#if defined(AFS_LINUX26_ENV) && defined(STRUCT_FILE_OPERATIONS_HAS_SPLICE)
+  .splice_write = generic_file_splice_write,
+  .splice_read = generic_file_splice_read,
 #endif
   .release =	afs_linux_release,
   .fsync =	afs_linux_fsync,
   .lock =	afs_linux_lock,
+#ifdef STRUCT_FILE_OPERATIONS_HAS_FLOCK
+  .flock =	afs_linux_flock,
+#endif
 };
 
 
@@ -793,7 +698,6 @@ check_bad_parent(struct dentry *dp)
     struct vcache *pvc = VTOAFS(dp->d_parent->d_inode);
 
     if (vcp->mvid->Fid.Volume != pvc->fid.Fid.Volume) {	/* bad parent */
-printk("check_bad_parent(%s): bad parent vcp->mvid->Fid.Volume != pvc->fid.Fid.Volume\n", dp->d_name.name);
 	credp = crref();
 
 	/* force a lookup, so vcp->mvid is fixed up */
@@ -828,6 +732,7 @@ afs_linux_revalidate(struct dentry *dp)
 #endif
     AFS_GLOCK();
 
+#ifdef notyet
     /* Make this a fast path (no crref), since it's called so often. */
     if (vcp->states & CStatd) {
 
@@ -840,11 +745,12 @@ afs_linux_revalidate(struct dentry *dp)
 #endif
 	return 0;
     }
+#endif
 
     credp = crref();
     code = afs_getattr(vcp, &vattr, credp);
     if (!code)
-        vattr2inode(AFSTOV(vcp), &vattr);
+        afs_fill_inode(AFSTOV(vcp), &vattr);
 
     AFS_GUNLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -889,11 +795,13 @@ afs_linux_dentry_revalidate(struct dentry *dp)
     cred_t *credp = NULL;
     struct vcache *vcp, *pvcp, *tvc = NULL;
     int valid;
+    struct afs_fakestat_state fakestate;
 
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
 #endif
     AFS_GLOCK();
+    afs_InitFakeStat(&fakestate);
 
     if (dp->d_inode) {
 
@@ -903,8 +811,34 @@ afs_linux_dentry_revalidate(struct dentry *dp)
 	if (vcp == afs_globalVp)
 	    goto good_dentry;
 
-	if (*dp->d_name.name != '/' && vcp->mvstat == 2)	/* root vnode */
-	    check_bad_parent(dp);	/* check and correct mvid */
+	if (vcp->mvstat == 1) {         /* mount point */
+	    if (vcp->mvid && (vcp->states & CMValid)) {
+		int tryEvalOnly = 0;
+		int code = 0;
+		struct vrequest treq;
+
+		credp = crref();
+		code = afs_InitReq(&treq, credp);
+		if (
+#ifdef AFS_DARWIN_ENV
+		    (strcmp(dp->d_name.name, ".DS_Store") == 0) ||
+		    (strcmp(dp->d_name.name, "Contents") == 0) ||
+#endif
+		    (strcmp(dp->d_name.name, ".directory") == 0)) {
+		    tryEvalOnly = 1;
+		}
+		if (tryEvalOnly)
+		    code = afs_TryEvalFakeStat(&vcp, &fakestate, &treq);
+		else
+		    code = afs_EvalFakeStat(&vcp, &fakestate, &treq);
+		if ((tryEvalOnly && vcp->mvstat == 1) || code) {
+		    /* a mount point, not yet replaced by its directory */
+		    goto bad_dentry;
+		}
+	    }
+	} else
+	    if (*dp->d_name.name != '/' && vcp->mvstat == 2) /* root vnode */
+		check_bad_parent(dp);	/* check and correct mvid */
 
 #ifdef notdef
 	/* If the last looker changes, we should make sure the current
@@ -964,6 +898,7 @@ afs_linux_dentry_revalidate(struct dentry *dp)
     /* Clean up */
     if (tvc)
 	afs_PutVCache(tvc);
+    afs_PutFakeStat(&fakestate);
     AFS_GUNLOCK();
     if (credp)
 	crfree(credp);
@@ -978,7 +913,10 @@ afs_linux_dentry_revalidate(struct dentry *dp)
     return valid;
 
   bad_dentry:
-    valid = 0;
+    if (have_submounts(dp))
+	valid = 1;
+    else 
+	valid = 0;
     goto done;
 }
 
@@ -988,9 +926,17 @@ afs_dentry_iput(struct dentry *dp, struct inode *ip)
     struct vcache *vcp = VTOAFS(ip);
 
     AFS_GLOCK();
-    if (vcp->states & CUnlinked)
-	(void) afs_remunlink(vcp, 1);		/* perhaps afs_InactiveVCache() instead */
+    (void) afs_InactiveVCache(vcp, NULL);
     AFS_GUNLOCK();
+#ifdef DCACHE_NFSFS_RENAMED
+#ifdef AFS_LINUX26_ENV
+    spin_lock(&dp->d_lock);
+#endif
+    dp->d_flags &= ~DCACHE_NFSFS_RENAMED;   
+#ifdef AFS_LINUX26_ENV
+    spin_unlock(&dp->d_lock);
+#endif
+#endif
 
     iput(ip);
 }
@@ -1052,6 +998,7 @@ afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
 
 	afs_getattr(vcp, &vattr, credp);
 	afs_fill_inode(ip, &vattr);
+	insert_inode_hash(ip);
 	dp->d_op = &afs_dentry_operations;
 	dp->d_time = hgetlo(VTOAFS(dip)->m.DataVersion);
 	d_instantiate(dp, ip);
@@ -1079,11 +1026,9 @@ static int
 afs_linux_lookup(struct inode *dip, struct dentry *dp)
 #endif
 {
-    struct vattr vattr;
     cred_t *credp = crref();
     struct vcache *vcp = NULL;
     const char *comp = dp->d_name.name;
-    struct dentry *res = NULL;
     struct inode *ip = NULL;
     int code;
 
@@ -1094,23 +1039,44 @@ afs_linux_lookup(struct inode *dip, struct dentry *dp)
     code = afs_lookup(VTOAFS(dip), comp, &vcp, credp);
     
     if (vcp) {
-	ip = AFSTOV(vcp);
+	struct vattr vattr;
 
+	ip = AFSTOV(vcp);
 	afs_getattr(vcp, &vattr, credp);
 	afs_fill_inode(ip, &vattr);
+	if (
+#ifdef HAVE_KERNEL_HLIST_UNHASHED
+	    hlist_unhashed(&ip->i_hash)
+#elif defined(AFS_LINUX26_ENV)
+	    ip->i_hash.pprev == NULL
+#else
+	    ip->i_hash.prev == NULL
+#endif
+	    )
+	    insert_inode_hash(ip);
     }
     dp->d_op = &afs_dentry_operations;
     dp->d_time = hgetlo(VTOAFS(dip)->m.DataVersion);
     AFS_GUNLOCK();
+
 #if defined(AFS_LINUX24_ENV)
     if (ip && S_ISDIR(ip->i_mode)) {
-            d_prune_aliases(ip);
-            res = d_find_alias(ip);
+	struct dentry *alias;
+
+	alias = d_find_alias(ip);
+	if (alias) {
+	    if (d_invalidate(alias) == 0) {
+		dput(alias);
+	    } else {
+		iput(ip);
+#if defined(AFS_LINUX26_ENV)
+		unlock_kernel();
+#endif
+		crfree(credp);
+		return alias;
+	    }
+	}
     }
-    if (res) {
-	if (d_unhashed(res))
-	    d_rehash(res);
-    } else
 #endif
     d_add(dp, ip);
 
@@ -1122,15 +1088,13 @@ afs_linux_lookup(struct inode *dip, struct dentry *dp)
     /* It's ok for the file to not be found. That's noted by the caller by
      * seeing that the dp->d_inode field is NULL.
      */
-#if defined(AFS_LINUX24_ENV)
-    if (code == 0)
-        return res;
-#endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,10)
     if (code == ENOENT)
 	return ERR_PTR(0);
-    else
+    else if ((code >= 0) && (code <= MAX_ERRNO))
 	return ERR_PTR(-code);
+    else 
+	return ERR_PTR(-EIO);
 #else
     if (code == ENOENT)
 	code = 0;
@@ -1170,7 +1134,7 @@ afs_linux_unlink(struct inode *dip, struct dentry *dp)
 #if defined(AFS_LINUX26_ENV)
     lock_kernel();
 #endif
-    if (((VREFCOUNT(tvc) > 0) && tvc->opens > 0)
+    if (VREFCOUNT(tvc) > 1 && tvc->opens > 0
 				&& !(tvc->states & CUnlinked)) {
 	struct dentry *__dp;
 	char *__name;
@@ -1203,6 +1167,17 @@ afs_linux_unlink(struct inode *dip, struct dentry *dp)
             }
             tvc->uncred = credp;
 	    tvc->states |= CUnlinked;
+#ifdef DCACHE_NFSFS_RENAMED
+#ifdef AFS_LINUX26_ENV
+	    spin_lock(&dp->d_lock);
+#endif
+	    dp->d_flags |= DCACHE_NFSFS_RENAMED;   
+#ifdef AFS_LINUX26_ENV
+	    spin_unlock(&dp->d_lock);
+#endif
+#endif
+	} else {
+	    osi_FreeSmallSpace(__name);	
 	}
 	AFS_GUNLOCK();
 
@@ -1294,9 +1269,8 @@ afs_linux_rmdir(struct inode *dip, struct dentry *dp)
     cred_t *credp = crref();
     const char *name = dp->d_name.name;
 
-#if defined(AFS_LINUX26_ENV)
-    lock_kernel();
-#endif
+    /* locking kernel conflicts with glock? */
+
     AFS_GLOCK();
     code = afs_rmdir(VTOAFS(dip), name, credp);
     AFS_GUNLOCK();
@@ -1313,9 +1287,6 @@ afs_linux_rmdir(struct inode *dip, struct dentry *dp)
 	d_drop(dp);
     }
 
-#if defined(AFS_LINUX26_ENV)
-    unlock_kernel();
-#endif
     crfree(credp);
     return -code;
 }
@@ -1334,13 +1305,7 @@ afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 #if defined(AFS_LINUX26_ENV)
     /* Prevent any new references during rename operation. */
     lock_kernel();
-#endif
-    /* Remove old and new entries from name hash. New one will change below.
-     * While it's optimal to catch failures and re-insert newdp into hash,
-     * it's also error prone and in that case we're already dealing with error
-     * cases. Let another lookup put things right, if need be.
-     */
-#if defined(AFS_LINUX26_ENV)
+
     if (!d_unhashed(newdp)) {
 	d_drop(newdp);
 	rehash = newdp;
@@ -1361,6 +1326,9 @@ afs_linux_rename(struct inode *oldip, struct dentry *olddp,
     code = afs_rename(VTOAFS(oldip), oldname, VTOAFS(newip), newname, credp);
     AFS_GUNLOCK();
 
+    if (!code)
+	olddp->d_time = 0;      /* force to revalidate */
+
     if (rehash)
 	d_rehash(rehash);
 
@@ -1395,7 +1363,7 @@ afs_linux_ireadlink(struct inode *ip, char *target, int maxlen, uio_seg_t seg)
 	return -code;
 }
 
-#if !defined(AFS_LINUX24_ENV)
+#if !defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
 /* afs_linux_readlink 
  * Fill target (which is in user space) with contents of symlink.
  */
@@ -1415,6 +1383,36 @@ afs_linux_readlink(struct dentry *dp, char *target, int maxlen)
 /* afs_linux_follow_link
  * a file system dependent link following routine.
  */
+#if defined(AFS_LINUX24_ENV)
+static int afs_linux_follow_link(struct dentry *dentry, struct nameidata *nd)
+{
+    int code;
+    char *name;
+
+    name = osi_Alloc(PATH_MAX);
+    if (!name) {
+	return -EIO;
+    }
+
+    AFS_GLOCK();
+    code = afs_linux_ireadlink(dentry->d_inode, name, PATH_MAX - 1, AFS_UIOSYS);
+    AFS_GUNLOCK();
+
+    if (code < 0) {
+	goto out;
+    }
+
+    name[code] = '\0';
+    code = vfs_follow_link(nd, name);
+
+out:
+    osi_Free(name, PATH_MAX);
+
+    return code;
+}
+
+#else /* !defined(AFS_LINUX24_ENV) */
+
 static struct dentry *
 afs_linux_follow_link(struct dentry *dp, struct dentry *basep,
 		      unsigned int follow)
@@ -1437,7 +1435,10 @@ afs_linux_follow_link(struct dentry *dp, struct dentry *basep,
 
     if (code < 0) {
 	dput(basep);
-	res = ERR_PTR(code);
+	if (code < -MAX_ERRNO)
+	    res = ERR_PTR(-EIO);
+	else
+	    res = ERR_PTR(code);
     } else {
 	name[code] = '\0';
 	res = lookup_dentry(name, basep, follow);
@@ -1448,7 +1449,8 @@ afs_linux_follow_link(struct dentry *dp, struct dentry *basep,
     AFS_GUNLOCK();
     return res;
 }
-#endif
+#endif /* AFS_LINUX24_ENV */
+#endif /* USABLE_KERNEL_PAGE_SYMLINK_CACHE */
 
 /* afs_linux_readpage
  * all reads come through here. A strategy-like read call.
@@ -1460,7 +1462,7 @@ afs_linux_readpage(struct file *fp, struct page *pp)
     cred_t *credp = crref();
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
     char *address;
-    afs_offs_t offset = pp->index << PAGE_CACHE_SHIFT;
+    afs_offs_t offset = ((loff_t) pp->index) << PAGE_CACHE_SHIFT;
 #else
     ulong address = afs_linux_page_address(pp);
     afs_offs_t offset = pageoff(pp);
@@ -1481,7 +1483,7 @@ afs_linux_readpage(struct file *fp, struct page *pp)
     clear_bit(PG_error, &pp->flags);
 #endif
 
-    setup_uio(&tuio, &iovec, (char *)address, offset, PAGESIZE, UIO_READ,
+    setup_uio(&tuio, &iovec, (char *)address, offset, PAGE_SIZE, UIO_READ,
 	      AFS_UIOSYS);
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
@@ -1499,7 +1501,7 @@ afs_linux_readpage(struct file *fp, struct page *pp)
 
     if (!code) {
 	if (tuio.uio_resid)	/* zero remainder of page */
-	    memset((void *)(address + (PAGESIZE - tuio.uio_resid)), 0,
+	    memset((void *)(address + (PAGE_SIZE - tuio.uio_resid)), 0,
 		   tuio.uio_resid);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
 	flush_dcache_page(pp);
@@ -1556,7 +1558,7 @@ afs_linux_writepage_sync(struct inode *ip, struct page *pp,
     int f_flags = 0;
 
     buffer = kmap(pp) + offset;
-    base = (pp->index << PAGE_CACHE_SHIFT) + offset;
+    base = (((loff_t) pp->index) << PAGE_CACHE_SHIFT)  + offset;
 
     credp = crref();
     lock_kernel();
@@ -1565,16 +1567,34 @@ afs_linux_writepage_sync(struct inode *ip, struct page *pp,
 	       ICL_TYPE_POINTER, pp, ICL_TYPE_INT32, page_count(pp),
 	       ICL_TYPE_INT32, 99999);
 
+    ObtainReadLock(&vcp->lock);
+    if (vcp->states & CPageWrite) {
+	ReleaseReadLock(&vcp->lock);
+	AFS_GUNLOCK();
+	unlock_kernel();
+	crfree(credp);
+	kunmap(pp);
+#ifdef AFS_LINUX26_ENV
+#if defined(WRITEPAGE_ACTIVATE)
+	return WRITEPAGE_ACTIVATE;
+#else
+	return AOP_WRITEPAGE_ACTIVATE;
+#endif
+#else
+	/* should mark it dirty? */
+	return(0); 
+#endif
+    }
+    ReleaseReadLock(&vcp->lock);
+
     setup_uio(&tuio, &iovec, buffer, base, count, UIO_WRITE, AFS_UIOSYS);
 
     code = afs_write(vcp, &tuio, f_flags, credp, 0);
 
-    ip->i_size = vcp->m.Length;
+    i_size_write(ip, vcp->m.Length);
     ip->i_blocks = ((vcp->m.Length + 1023) >> 10) << 1;
 
-    if (!code
-	&& afs_stats_cmperf.cacheCurrDirtyChunks >
-	afs_stats_cmperf.cacheMaxDirtyChunks) {
+    if (!code) {
 	struct vrequest treq;
 
 	ObtainWriteLock(&vcp->lock, 533);
@@ -1612,7 +1632,11 @@ afs_linux_writepage(struct page *pp)
 
 #if defined(AFS_LINUX26_ENV)
     if (PageReclaim(pp)) {
+# if defined(WRITEPAGE_ACTIVATE)
 	return WRITEPAGE_ACTIVATE;
+# else 
+	return AOP_WRITEPAGE_ACTIVATE;
+# endif
     }
 #else
     if (PageLaunder(pp)) {
@@ -1621,13 +1645,13 @@ afs_linux_writepage(struct page *pp)
 #endif
 
     inode = (struct inode *)mapping->host;
-    end_index = inode->i_size >> PAGE_CACHE_SHIFT;
+    end_index = i_size_read(inode) >> PAGE_CACHE_SHIFT;
 
     /* easy case */
     if (pp->index < end_index)
 	goto do_it;
     /* things got complicated... */
-    offset = inode->i_size & (PAGE_CACHE_SIZE - 1);
+    offset = i_size_read(inode) & (PAGE_CACHE_SIZE - 1);
     /* OK, are we completely out? */
     if (pp->index >= end_index + 1 || !offset)
 	return -EIO;
@@ -1670,9 +1694,19 @@ afs_linux_updatepage(struct file *fp, struct page *pp, unsigned long offset,
 
     code = afs_write(vcp, &tuio, fp->f_flags, credp, 0);
 
-    ip->i_size = vcp->m.Length;
+    i_size_write(ip, vcp->m.Length);
     ip->i_blocks = ((vcp->m.Length + 1023) >> 10) << 1;
 
+    if (!code) {
+	struct vrequest treq;
+
+	ObtainWriteLock(&vcp->lock, 533);
+	vcp->m.Date = osi_Time();   /* set modification time */
+	if (!afs_InitReq(&treq, credp))
+	    code = afs_DoPartialWrite(vcp, &treq);
+	ReleaseWriteLock(&vcp->lock);
+    }
+
     code = code ? -code : count - tuio.uio_resid;
     afs_Trace4(afs_iclSetp, CM_TRACE_UPDATEPAGE, ICL_TYPE_POINTER, vcp,
 	       ICL_TYPE_POINTER, pp, ICL_TYPE_INT32, page_count(pp),
@@ -1714,7 +1748,7 @@ afs_linux_permission(struct inode *ip, int mode)
     return -code;
 }
 
-#if defined(AFS_LINUX24_ENV)
+#if defined(AFS_LINUX24_ENV) && !(LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28) && defined(HAVE_WRITE_BEGIN))
 static int
 afs_linux_commit_write(struct file *file, struct page *page, unsigned offset,
 		       unsigned to)
@@ -1741,8 +1775,41 @@ afs_linux_prepare_write(struct file *file, struct page *page, unsigned from,
 #endif
     return 0;
 }
+#endif
 
-extern int afs_notify_change(struct dentry *dp, struct iattr *iattrp);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28) && defined(HAVE_WRITE_BEGIN)
+static int
+afs_linux_write_end(struct file *file, struct address_space *mapping,
+                                loff_t pos, unsigned len, unsigned copied,
+                                struct page *page, void *fsdata)
+{
+    int code;
+    pgoff_t index = pos >> PAGE_CACHE_SHIFT;
+    unsigned from = pos & (PAGE_CACHE_SIZE - 1);
+
+    code = afs_linux_writepage_sync(file->f_dentry->d_inode, page,
+                                    from, copied);
+    unlock_page(page);
+    page_cache_release(page);
+    return code;
+}
+
+static int
+afs_linux_write_begin(struct file *file, struct address_space *mapping,
+                                loff_t pos, unsigned len, unsigned flags,
+                                struct page **pagep, void **fsdata)
+{
+    struct page *page;
+    pgoff_t index = pos >> PAGE_CACHE_SHIFT;
+#if defined(HAVE_GRAB_CACHE_PAGE_WRITE_BEGIN)
+    page = grab_cache_page_write_begin(mapping, index, flags);
+#else
+    page = __grab_cache_page(mapping, index);
+#endif
+    *pagep = page;
+
+    return 0;
+}
 #endif
 
 static struct inode_operations afs_file_iops = {
@@ -1766,8 +1833,13 @@ static struct inode_operations afs_file_iops = {
 static struct address_space_operations afs_file_aops = {
   .readpage =		afs_linux_readpage,
   .writepage =		afs_linux_writepage,
-  .commit_write =	afs_linux_commit_write,
-  .prepare_write =	afs_linux_prepare_write,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,28) && defined(HAVE_WRITE_BEGIN)
+  .write_begin =        afs_linux_write_begin,
+  .write_end =          afs_linux_write_end,
+#else
+  .commit_write =       afs_linux_commit_write,
+  .prepare_write =      afs_linux_prepare_write,
+#endif
 };
 #endif
 
@@ -1801,7 +1873,7 @@ static struct inode_operations afs_dir_iops = {
 /* We really need a separate symlink set of ops, since do_follow_link()
  * determines if it _is_ a link by checking if the follow_link op is set.
  */
-#if defined(AFS_LINUX24_ENV)
+#if defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
 static int
 afs_symlink_filler(struct file *file, struct page *page)
 {
@@ -1836,10 +1908,10 @@ afs_symlink_filler(struct file *file, struct page *page)
 static struct address_space_operations afs_symlink_aops = {
   .readpage =	afs_symlink_filler
 };
-#endif
+#endif	/* USABLE_KERNEL_PAGE_SYMLINK_CACHE */
 
 static struct inode_operations afs_symlink_iops = {
-#if defined(AFS_LINUX24_ENV)
+#if defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
   .readlink = 		page_readlink,
 #if defined(HAVE_KERNEL_PAGE_FOLLOW_LINK)
   .follow_link =	page_follow_link,
@@ -1847,13 +1919,17 @@ static struct inode_operations afs_symlink_iops = {
   .follow_link =	page_follow_link_light,
   .put_link =           page_put_link,
 #endif
-  .setattr =		afs_notify_change,
-#else
+#else /* !defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE) */
   .readlink = 		afs_linux_readlink,
   .follow_link =	afs_linux_follow_link,
+#if !defined(AFS_LINUX24_ENV)
   .permission =		afs_linux_permission,
   .revalidate =		afs_linux_revalidate,
 #endif
+#endif /* USABLE_KERNEL_PAGE_SYMLINK_CACHE */
+#if defined(AFS_LINUX24_ENV)
+  .setattr =		afs_notify_change,
+#endif
 };
 
 void
@@ -1863,6 +1939,9 @@ afs_fill_inode(struct inode *ip, struct vattr *vattr)
     if (vattr)
 	vattr2inode(ip, vattr);
 
+#if defined(AFS_LINUX26_ENV)
+    ip->i_mapping->backing_dev_info = &afs_backing_dev_info;
+#endif
 /* Reset ops if symlink or directory. */
     if (S_ISREG(ip->i_mode)) {
 	ip->i_op = &afs_file_iops;
@@ -1879,11 +1958,10 @@ afs_fill_inode(struct inode *ip, struct vattr *vattr)
 
     } else if (S_ISLNK(ip->i_mode)) {
 	ip->i_op = &afs_symlink_iops;
-#if defined(AFS_LINUX24_ENV)
+#if defined(USABLE_KERNEL_PAGE_SYMLINK_CACHE)
 	ip->i_data.a_ops = &afs_symlink_aops;
 	ip->i_mapping = &ip->i_data;
 #endif
     }
 
-    /* insert_inode_hash(ip);	-- this would make iget() work (if we used it) */
 }
