@@ -16,15 +16,13 @@
  * afs_osi.h.
  */
 
-/* $Id: osi_machdep.h,v 1.16.2.4 2005/04/24 23:40:44 shadow Exp $ */
+/* $Id: osi_machdep.h,v 1.16.2.10 2009/03/27 15:55:45 shadow Exp $ */
 
 #ifndef _OSI_MACHDEP_H_
 #define _OSI_MACHDEP_H_
 
 #include <sys/lock.h>
 
-extern struct simplelock afs_rxglobal_lock;
-
 #define M_AFSFID	(M_TEMP-1)
 #define M_AFSBUFHDR	(M_TEMP-2)
 #define M_AFSBUFFER	(M_TEMP-3)
@@ -56,8 +54,35 @@ extern struct simplelock afs_rxglobal_lock;
 #define AFS_UIOUSER	UIO_USERSPACE
 
 /* malloc */
-#define AFS_KALLOC(s)	afs_nbsd_Alloc(s)
-#define AFS_KFREE(p, s)	afs_nbsd_Free((p), (s))
+extern void *osi_obsd_Alloc(size_t asize, int cansleep);
+extern void osi_obsd_Free(void *p, size_t asize);
+
+#ifdef afs_osi_Alloc_NoSleep
+#undef afs_osi_Alloc_NoSleep
+#define afs_osi_Alloc_NoSleep(asize) osi_obsd_Alloc((asize), 0)
+#endif
+
+#ifdef AFS_KALLOC
+#undef AFS_KALLOC
+#define AFS_KALLOC(s) osi_obsd_Alloc((s), 1 /* cansleep */)
+#endif
+
+#ifdef AFS_KFREE
+#undef AFS_KFREE
+#define AFS_KFREE(p, s) (osi_obsd_Free((p), (s)))
+#endif
+
+#ifdef AFS_OBSD42_ENV
+/* removed, live with it */
+#define BSD_KMALLOC(p, ptype, msize, mtype, mflags)     \
+  (p) = malloc((msize), (mtype), (mflags))
+
+#define BSD_KFREE(p, mflags) \
+  free((p), (mflags))
+#else
+#define BSD_KMALLOC MALLOC
+#define BSD_KFREE KFREE
+#endif /* AFS_OBSD42_ENV */
 
 /* proc, cred */
 #define	AFS_PROC	struct proc
@@ -69,10 +94,9 @@ extern struct simplelock afs_rxglobal_lock;
 #define p_rcred         p_ucred
 
 /* time */
-extern struct timeval time;
 #define	afs_hz		hz
 #define osi_GetTime(x)	microtime(x)
-#define osi_Time()	(time.tv_sec)
+extern time_t osi_Time();
 
 /* str */
 #define afs_strcasecmp(s1, s2)	strncasecmp((s1), (s2), 65535)
@@ -88,7 +112,6 @@ extern struct timeval time;
 #ifndef iodone
 #define iodone biodone
 #endif
-#define PAGESIZE	8192
 #define printk		printf	/* for RX version of xdr_* */
 #define setgroups	sys_setgroups
 #define UVM
@@ -96,45 +119,98 @@ extern struct timeval time;
 /* This is not always in scope yet */
 struct vcache;
 
-extern int afs_nbsd_lookupname(char *fnamep, enum uio_seg segflg,
+extern int afs_obsd_lookupname(char *fnamep, enum uio_seg segflg,
 			       int followlink, struct vnode **compvpp);
-extern void afs_nbsd_getnewvnode(struct vcache *tvc);
-extern void *afs_nbsd_Alloc(size_t asize);
-extern void afs_nbsd_Free(void *p, size_t asize);
+extern void afs_obsd_getnewvnode(struct vcache *tvc);
+extern void *afs_obsd_Alloc(size_t asize);
+extern void afs_obsd_Free(void *p, size_t asize);
 extern int afs_vget();
 
 #undef gop_lookupname
 #define	gop_lookupname(fnamep, segflg, followlink, compvpp) \
 	afs_nbsd_lookupname((fnamep), (segflg), (followlink), (compvpp))
 
+#ifdef AFS_OBSD39_ENV
+#define afs_osi_lockmgr(l, f, i, p) lockmgr((l), (f), (i))
+#else
+#define afs_osi_lockmgr(l, f, i, p) lockmgr((l), (f), (i), (p))
+#endif
+
 #ifdef KERNEL
 
-#ifdef AFS_GLOBAL_SUNLOCK
-extern struct proc *afs_global_owner;
+#ifdef AFS_OBSD44_ENV
+/* Revert to classical, BSD locks */
+
 extern struct lock afs_global_lock;
+extern struct proc *afs_global_owner;
+
+#ifdef AFS_GLOBAL_SUNLOCK
+
+#if defined(LOCKDEBUG)
+
 #define AFS_GLOCK() \
-    do { \
-        osi_Assert(curproc); \
- 	lockmgr(&afs_global_lock, LK_EXCLUSIVE, 0, curproc); \
-        osi_Assert(afs_global_owner == NULL); \
-   	afs_global_owner = curproc; \
-    } while (0)
+  do { \
+  _lockmgr(&afs_global_lock, LK_EXCLUSIVE, NULL, __FILE__, __LINE__); \
+  } while(0);
 #define AFS_GUNLOCK() \
-    do { \
-        osi_Assert(curproc); \
- 	osi_Assert(afs_global_owner == curproc); \
-        afs_global_owner = NULL; \
-        lockmgr(&afs_global_lock, LK_RELEASE, 0, curproc); \
-    } while(0)
-#define ISAFS_GLOCK() (afs_global_owner == curproc && curproc)
+  do { \
+  _lockmgr(&afs_global_lock, LK_RELEASE, NULL, __FILE__, __LINE__); \
+  } while(0);
+
 #else
-extern struct simplelock afs_global_lock;
-#define AFS_GLOCK()
-#define AFS_GUNLOCK()
+
+#define AFS_GLOCK() \
+  do { \
+  lockmgr(&afs_global_lock, LK_EXCLUSIVE, NULL); \
+  } while(0);
+#define AFS_GUNLOCK() \
+  do { \
+  lockmgr(&afs_global_lock, LK_RELEASE, NULL); \
+  } while(0);
+#endif /* LOCKDEBUG */
+#define ISAFS_GLOCK() (lockstatus(&afs_global_lock) == LK_EXCLUSIVE)
+#else
+extern struct lock afs_global_lock;
+#define AFS_GLOCKP(p)
+#define AFS_GUNLOCKP(p)
 #define AFS_ASSERT_GLOCK()
 #define ISAFS_GLOCK() 1
 #endif
 
+#else
+/* I don't see doing locks this way for older kernels, either,
+ * but, smart folks wrote this
+ */
+#define AFS_GLOCK() AFS_GLOCKP(curproc)
+#define AFS_GUNLOCK() AFS_GUNLOCKP(curproc)
+#ifdef AFS_GLOBAL_SUNLOCK
+extern struct proc *afs_global_owner;
+extern struct lock afs_global_lock;
+#define AFS_GLOCKP(p) \
+    do { \
+        osi_Assert(p); \
+ 	afs_osi_lockmgr(&afs_global_lock, LK_EXCLUSIVE, 0, (p)); \
+        osi_Assert(afs_global_owner == NULL); \
+   	afs_global_owner = (p); \
+    } while (0)
+#define AFS_GUNLOCKP(p) \
+    do { \
+        osi_Assert(p); \
+ 	osi_Assert(afs_global_owner == (p)); \
+        afs_global_owner = NULL; \
+        afs_osi_lockmgr(&afs_global_lock, LK_RELEASE, 0, (p)); \
+    } while(0)
+#define ISAFS_GLOCK() (afs_global_owner == curproc && curproc)
+#else
+extern struct lock afs_global_lock;
+#define AFS_GLOCKP(p)
+#define AFS_GUNLOCKP(p)
+#define AFS_ASSERT_GLOCK()
+#define ISAFS_GLOCK() 1
+#endif
+
+#endif /* AFS_OBSD44_ENV */
+
 #undef SPLVAR
 #define SPLVAR int splvar
 #undef NETPRI
