@@ -18,7 +18,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.50.2.10 2005/07/21 04:49:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.50.2.22 2008/08/26 14:02:14 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -249,7 +249,7 @@ afs_InitFakeStat(struct afs_fakestat_state *state)
  *
  * Only issues RPCs if canblock is non-zero.
  */
-int
+static int
 afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 		     struct vrequest *areq, int canblock)
 {
@@ -291,7 +291,7 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 	    do {
 		retry = 0;
 		ObtainWriteLock(&afs_xvcache, 597);
-		root_vp = afs_FindVCache(tvc->mvid, &retry, 0);
+		root_vp = afs_FindVCache(tvc->mvid, &retry, IS_WLOCK);
 		if (root_vp && retry) {
 		    ReleaseWriteLock(&afs_xvcache);
 		    afs_PutVCache(root_vp);
@@ -305,6 +305,14 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 	    code = canblock ? ENOENT : 0;
 	    goto done;
 	}
+#ifdef AFS_DARWIN80_ENV
+        root_vp->m.Type = VDIR;
+        AFS_GUNLOCK();
+        code = afs_darwin_finalizevnode(root_vp, NULL, NULL, 0);
+        AFS_GLOCK();
+        if (code) goto done;
+        vnode_ref(AFSTOV(root_vp));
+#endif
 	if (tvolp) {
 	    /* Is this always kosher?  Perhaps we should instead use
 	     * NBObtainWriteLock to avoid potential deadlock.
@@ -489,7 +497,7 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 	    au = afs_GetUser(areq->uid, avc->fid.Cell, 0);
 	    if (au->exporter) {
 		error =
-		    EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, num);
+		    EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, &num);
 		if (error) {
 		    return 0;
 		}
@@ -530,8 +538,6 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     int nskip;			/* # of slots in the LRU queue to skip */
     struct vcache *lruvcp;	/* vcache ptr of our goal pos in LRU queue */
     struct dcache *dcp;		/* chunk containing the dir block */
-    char *statMemp;		/* status memory block */
-    char *cbfMemp;		/* callback and fid memory block */
     afs_size_t temp;		/* temp for holding chunk length, &c. */
     struct AFSFid *fidsp;	/* file IDs were collecting */
     struct AFSCallBack *cbsp;	/* call back pointers */
@@ -563,6 +569,9 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     int flagIndex = 0;		/* First file with bulk fetch flag set */
     int inlinebulk = 0;		/* Did we use InlineBulk RPC or not? */
     XSTATS_DECLS;
+#ifdef AFS_DARWIN80_ENV
+    panic("bulkstatus doesn't work on AFS_DARWIN80_ENV. don't call it");
+#endif
     /* first compute some basic parameters.  We dont want to prefetch more
      * than a fraction of the cache in any given call, and we want to preserve
      * a portion of the LRU queue in any event, so as to avoid thrashing
@@ -586,13 +595,11 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * one for fids and callbacks, and one for stat info.  Well set
      * up our pointers to the memory from there, too.
      */
-    statMemp = osi_AllocLargeSpace(nentries * sizeof(AFSFetchStatus));
-    statsp = (struct AFSFetchStatus *)statMemp;
-    cbfMemp =
-	osi_AllocLargeSpace(nentries *
-			    (sizeof(AFSCallBack) + sizeof(AFSFid)));
-    fidsp = (AFSFid *) cbfMemp;
-    cbsp = (AFSCallBack *) (cbfMemp + nentries * sizeof(AFSFid));
+    statsp = (AFSFetchStatus *) 
+	    osi_Alloc(AFSCBMAX * sizeof(AFSFetchStatus));
+    fidsp = (AFSFid *) osi_AllocLargeSpace(nentries * sizeof(AFSFid));
+    cbsp = (AFSCallBack *) 
+	    osi_Alloc(AFSCBMAX * sizeof(AFSCallBack));
 
     /* next, we must iterate over the directory, starting from the specified
      * cookie offset (dirCookie), and counting out nentries file entries.
@@ -690,7 +697,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    do {
 		retry = 0;
 		ObtainWriteLock(&afs_xvcache, 130);
-		tvcp = afs_FindVCache(&tfid, &retry, 0 /* no stats | LRU */ );
+		tvcp = afs_FindVCache(&tfid, &retry, IS_WLOCK /* no stats | LRU */ );
 		if (tvcp && retry) {
 		    ReleaseWriteLock(&afs_xvcache);
 		    afs_PutVCache(tvcp);
@@ -698,16 +705,36 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    } while (tvcp && retry);
 	    if (!tvcp) {	/* otherwise, create manually */
 		tvcp = afs_NewVCache(&tfid, hostp);
-		ObtainWriteLock(&tvcp->lock, 505);
-		ReleaseWriteLock(&afs_xvcache);
-		afs_RemoveVCB(&tfid);
-		ReleaseWriteLock(&tvcp->lock);
+		if (tvcp)
+		{
+			ObtainWriteLock(&tvcp->lock, 505);
+			ReleaseWriteLock(&afs_xvcache);
+			afs_RemoveVCB(&tfid);
+			ReleaseWriteLock(&tvcp->lock);
+		} else {
+			ReleaseWriteLock(&afs_xvcache);
+		}
 	    } else {
 		ReleaseWriteLock(&afs_xvcache);
 	    }
 	    if (!tvcp)
-		goto done;	/* can't happen at present, more's the pity */
+	    {
+		DRelease((struct buffer *)dirEntryp, 0);
+		ReleaseReadLock(&dcp->lock);
+		ReleaseReadLock(&adp->lock);
+		afs_PutDCache(dcp);
+		goto done;	/* can happen if afs_NewVCache fails */
+	    }
 
+#ifdef AFS_DARWIN80_ENV
+            if (tvcp->states & CVInit) {
+                 /* XXX don't have status yet, so creating the vnode is
+                    not yet useful. we would get CDeadVnode set, and the
+                    upcoming PutVCache will cause the vcache to be flushed &
+                    freed, which in turn means the bulkstatus results won't 
+                    be used */
+            }
+#endif
 	    /* WARNING: afs_DoBulkStat uses the Length field to store a
 	     * sequence number for each bulk status request. Under no
 	     * circumstances should afs_DoBulkStat store a sequence number
@@ -1050,7 +1077,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	afs_PutVolume(volp, READ_LOCK);
 
     /* If we did the InlineBulk RPC pull out the return code */
-    if (inlinebulk) {
+    if (inlinebulk && code == 0) {
 	if ((&statsp[0])->errorCode) {
 	    afs_Analyze(tcp, (&statsp[0])->errorCode, &adp->fid, areqp,
 			AFS_STATS_FS_RPCIDX_BULKSTATUS, SHARED_LOCK, NULL);
@@ -1060,13 +1087,18 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	code = 0;
     }
   done2:
-    osi_FreeLargeSpace(statMemp);
-    osi_FreeLargeSpace(cbfMemp);
+    osi_FreeLargeSpace((char *)fidsp);
+    osi_Free((char *)statsp, AFSCBMAX * sizeof(AFSFetchStatus));
+    osi_Free((char *)cbsp, AFSCBMAX * sizeof(AFSCallBack));
     return code;
 }
 
 /* was: (AFS_DEC_ENV) || defined(AFS_OSF30_ENV) || defined(AFS_NCR_ENV) */
+#ifdef AFS_DARWIN80_ENV
+#define AFSDOBULK 0
+#else
 static int AFSDOBULK = 1;
+#endif
 
 int
 #ifdef AFS_OSF_ENV
@@ -1085,6 +1117,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     register afs_int32 code;
     register afs_int32 bulkcode = 0;
     int pass = 0, hit = 0;
+    int force_eval = afs_fakestat_enable ? 0 : 1;
     long dirCookie;
     extern afs_int32 afs_mariner;	/*Writing activity to log? */
     afs_hyper_t versionNo;
@@ -1105,6 +1138,11 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     ndp->ni_dvp = AFSTOV(adp);
 #endif /* AFS_OSF_ENV */
 
+    if (afs_fakestat_enable && adp->mvstat == 1) {
+       if (strcmp(aname, ".directory") == 0)
+           tryEvalOnly = 1;
+    }
+
 #if defined(AFS_DARWIN_ENV)
     /* Workaround for MacOSX Finder, which tries to look for
      * .DS_Store and Contents under every directory.
@@ -1169,7 +1207,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	*avcp = tvc;
 	code = (tvc ? 0 : ENOENT);
 	hit = 1;
-	if (tvc && !VREFCOUNT(tvc)) {
+	if (tvc && !VREFCOUNT_GT(tvc, 0)) {
 	    osi_Panic("TT1");
 	}
 	if (code) {
@@ -1202,10 +1240,13 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	ObtainReadLock(&afs_xvcache);
 	osi_vnhold(adp, 0);
 	ReleaseReadLock(&afs_xvcache);
+#ifdef AFS_DARWIN80_ENV
+        vnode_get(AFSTOV(adp));
+#endif
 	code = 0;
 	*avcp = tvc = adp;
 	hit = 1;
-	if (adp && !VREFCOUNT(adp)) {
+	if (adp && !VREFCOUNT_GT(adp, 0)) {
 	    osi_Panic("TT2");
 	}
 	goto done;
@@ -1226,7 +1267,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     if (tvc) {
 	if (no_read_access && vType(tvc) != VDIR && vType(tvc) != VLNK) {
 	    /* need read access on dir to stat non-directory / non-link */
+#ifndef AFS_FBSD80_ENV
 	    afs_PutVCache(tvc);
+#endif
 	    *avcp = NULL;
 	    code = EACCES;
 	    goto done;
@@ -1372,7 +1415,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 
 	    /* if the vcache isn't usable, release it */
 	    if (tvc && !(tvc->states & CStatd)) {
-		afs_PutVCache(tvc);
+#ifndef  AFS_FBSD80_ENV
+	      afs_PutVCache(tvc);
+#endif
 		tvc = NULL;
 	    }
 	} else {
@@ -1399,8 +1444,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     }				/* sub-block just to reduce stack usage */
 
     if (tvc) {
-	int force_eval = afs_fakestat_enable ? 0 : 1;
-
 	if (adp->states & CForeign)
 	    tvc->states |= CForeign;
 	tvc->parentVnode = adp->fid.Fid.Vnode;
@@ -1421,6 +1464,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		force_eval = 1;
 	    ReleaseReadLock(&tvc->lock);
 	}
+	if (tvc->mvstat == 1 && (tvc->states & CMValid) && tvc->mvid != NULL)
+	  force_eval = 1; /* This is now almost for free, get it correct */
+
 #if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
 	if (!(flags & AFS_LOOKUP_NOEVAL))
 	    /* don't eval mount points */
@@ -1434,7 +1480,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		ReleaseWriteLock(&tvc->lock);
 
 		if (code) {
+#ifndef AFS_FBSD80_ENV
 		    afs_PutVCache(tvc);
+#endif
 		    if (tvolp)
 			afs_PutVolume(tvolp, WRITE_LOCK);
 		    goto done;
@@ -1456,7 +1504,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		    } else {
 			tvc = afs_GetVCache(tvc->mvid, &treq, NULL, NULL);
 		    }
+#ifndef AFS_FBSD80_ENV
 		    afs_PutVCache(uvc);	/* we're done with it */
+#endif
 
 		    if (!tvc) {
 			code = ENOENT;
@@ -1481,7 +1531,9 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 			afs_PutVolume(tvolp, WRITE_LOCK);
 		    }
 		} else {
+#ifndef AFS_FBSD80_ENV
 		    afs_PutVCache(tvc);
+#endif
 		    code = ENOENT;
 		    if (tvolp)
 			afs_PutVolume(tvolp, WRITE_LOCK);
@@ -1489,7 +1541,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 		}
 	    }
 	*avcp = tvc;
-	if (tvc && !VREFCOUNT(tvc)) {
+	if (tvc && !VREFCOUNT_GT(tvc, 0)) {
 	    osi_Panic("TT3");
 	}
 	code = 0;
@@ -1546,7 +1598,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
 	     * rather than the vc of the mount point itself.  we can still find the
 	     * mount point's vc in the vcache by its fid. */
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
-	    if (!hit) {
+	    if (!hit && force_eval) {
 		osi_dnlc_enter(adp, aname, tvc, &versionNo);
 	    } else {
 #ifdef AFS_LINUX20_ENV
@@ -1560,8 +1612,8 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct AFS_UCRED
     }
     if (bulkcode)
 	code = bulkcode;
-    else
-	code = afs_CheckCode(code, &treq, 19);
+
+    code = afs_CheckCode(code, &treq, 19);
     if (code) {
 	/* If there is an error, make sure *avcp is null.
 	 * Alphas panic otherwise - defect 10719.
