@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_osi.c,v 1.45 2004/06/21 20:01:50 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_osi.c,v 1.48.2.20 2009/03/27 15:55:43 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -58,9 +58,16 @@ osi_Init(void)
     usimple_lock_init(&afs_global_lock);
     afs_global_owner = (thread_t) 0;
 #elif defined(AFS_FBSD50_ENV)
+#if defined(AFS_FBSD80_ENV) && defined(WITNESS)
+    /* "lock_initalized" (sic) can panic, checks a flag bit
+     * is unset _before_ init */
+    memset(&afs_global_mtx, 0, sizeof(struct mtx));
+#endif
     mtx_init(&afs_global_mtx, "AFS global lock", NULL, MTX_DEF);
 #elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if !defined(AFS_DARWIN80_ENV)
     lockinit(&afs_global_lock, PLOCK, "afs global lock", 0, 0);
+#endif
     afs_global_owner = 0;
 #elif defined(AFS_AIX41_ENV)
     lock_alloc((void *)&afs_global_lock, LOCK_ALLOC_PIN, 1, 1);
@@ -69,11 +76,15 @@ osi_Init(void)
     /* Linux initialization in osi directory. Should move the others. */
     mutex_init(&afs_global_lock, "afs_global_lock", MUTEX_DEFAULT, NULL);
 #endif
-    /* afs_rxglobal_lock is initialized in rx_Init. */
 #endif /* AFS_GLOBAL_SUNLOCK */
 #endif /* AFS_HPUX_ENV */
 
     if (!afs_osicred_initialized) {
+#if defined(AFS_DARWIN80_ENV)
+        afs_osi_ctxtp_initialized = 0;
+        afs_osi_ctxtp = NULL; /* initialized in afs_Daemon since it has
+                                  a proc reference that cannot be changed */
+#endif
 #if defined(AFS_XBSD_ENV)
 	/* Can't just invent one, must use crget() because of mutex */
 	afs_osi_credp = crdup(osi_curcred());
@@ -82,7 +93,12 @@ osi_Init(void)
 #if defined(AFS_LINUX26_ENV)
         afs_osi_cred.cr_group_info = groups_alloc(0);
 #endif
+#if defined(AFS_DARWIN80_ENV)
+        afs_osi_cred.cr_ref = 1; /* kauth_cred_get_ref needs 1 existing ref */
+#else
 	crhold(&afs_osi_cred);	/* don't let it evaporate */
+#endif
+
 	afs_osi_credp = &afs_osi_cred;
 #endif
 	afs_osicred_initialized = 1;
@@ -98,13 +114,9 @@ int
 osi_Active(register struct vcache *avc)
 {
     AFS_STATCNT(osi_Active);
-#if defined(AFS_SUN_ENV) || defined(AFS_AIX_ENV) || defined(AFS_OSF_ENV) || defined(AFS_SUN5_ENV) || (AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_AIX_ENV) || defined(AFS_OSF_ENV) || defined(AFS_SUN5_ENV) || (AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     if ((avc->opens > 0) || (avc->states & CMAPPED))
 	return 1;		/* XXX: Warning, verify this XXX  */
-#elif	defined(AFS_MACH_ENV)
-    if (avc->opens > 0
-	|| ((avc->v.v_flag & VTEXT) && !inode_uncache_try(avc)))
-	return 1;
 #elif defined(AFS_SGI_ENV)
     if ((avc->opens > 0) || AFS_VN_MAPPED(AFSTOV(avc)))
 	return 1;
@@ -127,6 +139,7 @@ osi_Active(register struct vcache *avc)
 void
 osi_FlushPages(register struct vcache *avc, struct AFS_UCRED *credp)
 {
+    int vfslocked;
     afs_hyper_t origDV;
     ObtainReadLock(&avc->lock);
     /* If we've already purged this version, or if we're the ones
@@ -157,9 +170,19 @@ osi_FlushPages(register struct vcache *avc, struct AFS_UCRED *credp)
 	       ICL_TYPE_INT32, origDV.low, ICL_TYPE_INT32, avc->m.Length);
 
     ReleaseWriteLock(&avc->lock);
+#ifdef AFS_FBSD70_ENV
+    vfslocked = VFS_LOCK_GIANT(AFSTOV(avc)->v_mount);
+#endif
+#ifndef AFS_FBSD70_ENV
     AFS_GUNLOCK();
+#endif
     osi_VM_FlushPages(avc, credp);
+#ifndef AFS_FBSD70_ENV
     AFS_GLOCK();
+#endif
+#ifdef AFS_FBSD70_ENV
+    VFS_UNLOCK_GIANT(vfslocked);
+#endif
     ObtainWriteLock(&avc->lock, 88);
 
     /* do this last, and to original version, since stores may occur
@@ -189,14 +212,6 @@ osi_FlushText_really(register struct vcache *vp)
     if (hcmp(vp->m.DataVersion, vp->flushDV) <= 0)
 	return;
 
-#ifdef AFS_DEC_ENV
-    {
-	void afs_gfs_FlushText();
-	afs_gfs_FlushText(vp);
-	return;
-    }
-#else
-
     MObtainWriteLock(&afs_ftf, 317);
     hset(fdv, vp->m.DataVersion);
 
@@ -239,60 +254,7 @@ osi_FlushText_really(register struct vcache *vp)
     hset(vp->flushDV, fdv);
     MReleaseWriteLock(&afs_ftf);
 
-#endif /* AFS_DEC_ENV */
 }
-
-#ifdef AFS_DEC_ENV
-/* I don't really like using xinval() here, because it kills processes
- * a bit aggressively.  Previous incarnations of this functionality
- * used to use xrele() instead of xinval, and didn't invoke
- * cacheinval().  But they would panic.  So it might be worth looking
- * into some middle ground...
- */
-static void
-afs_gfs_FlushText(register struct vcache *vp)
-{
-    afs_hyper_t fdv;		/* version before which we'll flush */
-    register struct text *xp;
-    struct gnode *gp;
-
-    MObtainWriteLock(&afs_ftf, 318);
-    hset(fdv, vp->m.DataVersion);
-    gp = afs_vntogn(vp);
-
-    if (!gp) {
-	/* this happens frequently after cores are created. */
-	MReleaseWriteLock(&afs_ftf);
-	return;
-    }
-
-    if (gp->g_flag & GTEXT) {
-	if (gp->g_textp) {
-	    xp = (struct text *)gp->g_textp;
-	    /* if text object is locked, give up */
-	    if (xp && (xp->x_flag & XLOCK)) {
-		MReleaseWriteLock(&afs_ftf);
-		return;
-	    }
-	} else
-	    xp = NULL;
-
-	if (gp->g_flag & GTEXT) {	/* still has a text object? */
-	    xinval(gp);
-	}
-    }
-
-    /* next do the stuff that need not check for deadlock problems */
-    /*    maybe xinval(gp); here instead of above */
-    binval(NODEV, gp);
-    cacheinval(gp);
-    /* finally, record that we've done it */
-    hset(vp->flushDV, fdv);
-
-    MReleaseWriteLock(&afs_ftf);
-}
-#endif /* AFS_DEC_ENV */
-
 #endif /* AFS_TEXT_ENV */
 
 /* mask signals in afsds */
@@ -310,13 +272,30 @@ afs_osi_UnmaskRxkSignals(void)
 {
 }
 
+/* Two hacks to try and fix afsdb */
+void 
+afs_osi_MaskUserLoop()
+{
+#ifdef AFS_DARWIN_ENV
+    afs_osi_Invisible();
+    afs_osi_fullSigMask();
+#else
+    afs_osi_MaskSignals();
+#endif
+}
+
+void 
+afs_osi_UnmaskUserLoop()
+{
+#ifdef AFS_DARWIN_ENV
+    afs_osi_fullSigRestore();
+#endif
+}
+
 /* register rxk listener proc info */
 void
 afs_osi_RxkRegister(void)
 {
-#ifdef AFS_LINUX22_ENV
-    osi_linux_rxkreg();
-#endif
 }
 
 /* procedure for making our processes as invisible as we can */
@@ -325,12 +304,11 @@ afs_osi_Invisible(void)
 {
 #ifdef AFS_LINUX22_ENV
     afs_osi_MaskSignals();
-#elif defined(AFS_DEC_ENV)
-    u.u_procp->p_type |= SSYS;
 #elif defined(AFS_SUN5_ENV)
     curproc->p_flag |= SSYS;
 #elif defined(AFS_HPUX101_ENV) && !defined(AFS_HPUX1123_ENV)
     set_system_proc(u.u_procp);
+#elif defined(AFS_DARWIN80_ENV)
 #elif defined(AFS_DARWIN_ENV)
     /* maybe call init_process instead? */
     current_proc()->p_flag |= P_SYSTEM;
@@ -343,8 +321,21 @@ afs_osi_Invisible(void)
     AFS_STATCNT(osi_Invisible);
 }
 
+void
+afs_osi_Visible(void)
+{
+#if defined(AFS_SUN5_ENV)
+    curproc->p_flag &= ~SSYS;
+#elif defined(AFS_DARWIN80_ENV)
+#elif defined(AFS_DARWIN_ENV)
+    /* maybe call init_process instead? */
+    current_proc()->p_flag &= ~P_SYSTEM;
+#elif defined(AFS_XBSD_ENV)
+    curproc->p_flag &= ~P_SYSTEM;
+#endif
+}
 
-#if !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV)
+#if !defined(AFS_LINUX20_ENV) && !defined(AFS_XBSD_ENV)
 /* set the real time */
 void
 afs_osi_SetTime(osi_timeval_t * atv)
@@ -380,9 +371,11 @@ afs_osi_SetTime(osi_timeval_t * atv)
     stime(&sta);
     AFS_GLOCK();
 #elif defined(AFS_DARWIN_ENV)
+#ifndef AFS_DARWIN80_ENV
     AFS_GUNLOCK();
     setthetime(atv);
     AFS_GLOCK();
+#endif
 #else
     /* stolen from kern_time.c */
 #ifndef	AFS_AUX_ENV
@@ -425,8 +418,10 @@ afs_osi_SetTime(osi_timeval_t * atv)
 void *
 afs_osi_Alloc(size_t x)
 {
+#if !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV)
     register struct osimem *tm = NULL;
     register int size;
+#endif
 
     AFS_STATCNT(osi_Alloc);
     /* 0-length allocs may return NULL ptr from AFS_KALLOC, so we special-case
@@ -440,10 +435,12 @@ afs_osi_Alloc(size_t x)
     return osi_linux_alloc(x, 1);
 #elif defined(AFS_FBSD_ENV)
     return osi_fbsd_alloc(x, 1);
+#elif defined(AFS_OBSD_ENV)
+    return osi_obsd_Alloc(x, 1);
 #else
     size = x;
     tm = (struct osimem *)AFS_KALLOC(size);
-#ifdef	AFS_SUN_ENV
+#ifdef	AFS_SUN5_ENV
     if (!tm)
 	osi_Panic("osi_Alloc: Couldn't allocate %d bytes; out of memory!\n",
 		  size);
@@ -452,7 +449,7 @@ afs_osi_Alloc(size_t x)
 #endif
 }
 
-#if	defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV)
+#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 
 void *
 afs_osi_Alloc_NoSleep(size_t x)
@@ -555,7 +552,7 @@ osi_VMDirty_p(struct vcache *avc)
 #endif
 #endif /* AFS_AIX32_ENV */
 
-#if defined (AFS_SUN_ENV)
+#if defined (AFS_SUN5_ENV)
     if (avc->states & CMAPPED) {
 	struct page *pg;
 	for (pg = avc->v.v_s.v_Pages; pg; pg = pg->p_vpnext) {
@@ -605,6 +602,14 @@ void
 shutdown_osi(void)
 {
     AFS_STATCNT(shutdown_osi);
+#ifdef AFS_DARWIN80_ENV
+    if (afs_osi_ctxtp_initialized && afs_osi_ctxtp) {
+       vfs_context_rele(afs_osi_ctxtp);
+       afs_osi_ctxtp = NULL;
+       afs_osi_ctxtp_initialized = 0;
+    }
+    shutdown_osisleep();
+#endif
     if (afs_cold_shutdown) {
 	LOCK_INIT(&afs_ftf, "afs_ftf");
     }
@@ -612,12 +617,14 @@ shutdown_osi(void)
 
 #ifndef AFS_OBSD_ENV
 int
-afs_osi_suser(void *credp)
+afs_osi_suser(void *cr)
 {
-#if defined(AFS_SUN5_ENV)
-    return afs_suser(credp);
+#if defined(AFS_SUN510_ENV)
+    return (priv_policy(cr, PRIV_SYS_SUSER_COMPAT, B_FALSE, EPERM, NULL) == 0);
+#elif defined(AFS_SUN5_ENV)
+    return afs_suser(cr);
 #else
-    return afs_suser();
+    return afs_suser(NULL);
 #endif
 }
 #endif
@@ -786,6 +793,12 @@ afs_osi_TraverseProcTable(void)
 #endif
 
 #if defined(AFS_OSF_ENV)
+
+#ifdef AFS_DUX50_ENV
+extern struct pid_entry *pidtab;
+extern int npid; 
+#endif
+
 void
 afs_osi_TraverseProcTable(void)
 {
@@ -804,7 +817,7 @@ afs_osi_TraverseProcTable(void)
 }
 #endif
 
-#if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+#if (defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN80_ENV)) || defined(AFS_FBSD_ENV)
 void
 afs_osi_TraverseProcTable(void)
 {
@@ -822,29 +835,59 @@ afs_osi_TraverseProcTable(void)
 #endif
 
 #if defined(AFS_LINUX22_ENV)
+#ifdef EXPORTED_TASKLIST_LOCK
+extern rwlock_t tasklist_lock __attribute__((weak));
+#endif
 void
 afs_osi_TraverseProcTable()
 {
+#if !defined(LINUX_KEYRING_SUPPORT)
     struct task_struct *p;
 
-#ifdef EXPORTED_TASKLIST_LOCK
-    read_lock(&tasklist_lock);
-#endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18) && defined(EXPORTED_TASKLIST_LOCK)
+    if (&tasklist_lock)
+	read_lock(&tasklist_lock);
+#endif /* EXPORTED_TASKLIST_LOCK */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16) 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18) && defined(EXPORTED_TASKLIST_LOCK)
+    else
+#endif /* EXPORTED_TASKLIST_LOCK && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18) */
+	rcu_read_lock();
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16) */
+
 #ifdef DEFINED_FOR_EACH_PROCESS
     for_each_process(p) if (p->pid) {
+#ifdef STRUCT_TASK_STRUCT_HAS_EXIT_STATE
+	if (p->exit_state)
+	    continue;
+#else
 	if (p->state & TASK_ZOMBIE)
 	    continue;
+#endif
 	afs_GCPAGs_perproc_func(p);
     }
 #else
     for_each_task(p) if (p->pid) {
+#ifdef STRUCT_TASK_STRUCT_HAS_EXIT_STATE
+	if (p->exit_state)
+	    continue;
+#else
 	if (p->state & TASK_ZOMBIE)
 	    continue;
+#endif
 	afs_GCPAGs_perproc_func(p);
     }
 #endif
-#ifdef EXPORTED_TASKLIST_LOCK
-    read_unlock(&tasklist_lock);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18) && defined(EXPORTED_TASKLIST_LOCK)
+    if (&tasklist_lock)
+	read_unlock(&tasklist_lock);
+#endif /* EXPORTED_TASKLIST_LOCK */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16) 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18) && defined(EXPORTED_TASKLIST_LOCK)
+    else
+#endif /* EXPORTED_TASKLIST_LOCK && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18) */
+	rcu_read_unlock();
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16) */
 #endif
 }
 #endif
@@ -1020,6 +1063,25 @@ afs_osi_proc2cred(AFS_PROC * pr)
 
     return rv;
 }
+#elif defined(AFS_DARWIN80_ENV) 
+const struct AFS_UCRED *
+afs_osi_proc2cred(AFS_PROC * pr)
+{
+    struct AFS_UCRED *rv = NULL;
+    static struct AFS_UCRED cr;
+    struct ucred *pcred;
+
+    if (pr == NULL) {
+	return NULL;
+    }
+    pcred = proc_ucred(pr);
+    cr.cr_ref = 1;
+    cr.cr_uid = pcred->cr_uid;
+    cr.cr_ngroups = pcred->cr_ngroups;
+    memcpy(cr.cr_groups, pcred->cr_groups,
+           NGROUPS * sizeof(gid_t));
+    return &cr;
+}
 #elif defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 const struct AFS_UCRED *
 afs_osi_proc2cred(AFS_PROC * pr)
@@ -1060,10 +1122,15 @@ afs_osi_proc2cred(AFS_PROC * pr)
 	|| (pr->state == TASK_UNINTERRUPTIBLE)
 	|| (pr->state == TASK_STOPPED)) {
 	cr.cr_ref = 1;
-	cr.cr_uid = pr->uid;
+	cr.cr_uid = task_uid(pr);
 #if defined(AFS_LINUX26_ENV)
+#if defined(STRUCT_TASK_HAS_CRED)
+	get_group_info(pr->cred->group_info);
+	cr.cr_group_info = pr->cred->group_info;
+#else
 	get_group_info(pr->group_info);
 	cr.cr_group_info = pr->group_info;
+#endif
 #else
 	cr.cr_ngroups = pr->ngroups;
 	memcpy(cr.cr_groups, pr->groups, NGROUPS * sizeof(gid_t));
