@@ -11,11 +11,10 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/bucoord/tape_hosts.c,v 1.7 2003/07/15 23:14:47 shadow Exp $");
+    ("$Header: /cvs/openafs/src/bucoord/tape_hosts.c,v 1.7.2.4 2007/10/31 04:13:35 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
-#include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #else
@@ -50,10 +49,8 @@ extern char *whoami;
  *	Add a host to the tape hosts
  */
 
-afs_int32
-bc_AddHostCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_AddHostCmd(struct cmd_syndesc *as, void *arock)
 {
     struct cmd_item *ti;
     udbClientTextP ctPtr;
@@ -67,7 +64,7 @@ bc_AddHostCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -86,14 +83,14 @@ bc_AddHostCmd(as, arock)
 	    printf("failed\n");
 	    fflush(stdout);
 	    if (code == EEXIST)
-		com_err(whoami, 0, "Port offset already in tape database");
+		afs_com_err(whoami, 0, "Port offset already in tape database");
 	    ERROR(code);
 	}
 
 	code = bc_SaveHosts();
 	if (code) {
-	    com_err(whoami, code, "Cannot save tape hosts");
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, code, "Cannot save tape hosts");
+	    afs_com_err(whoami, 0,
 		    "Changes are temporary - for this session only");
 	    ERROR(code);
 	}
@@ -108,10 +105,8 @@ bc_AddHostCmd(as, arock)
     return (code);
 }
 
-afs_int32
-bc_DeleteHostCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_DeleteHostCmd(struct cmd_syndesc *as, void *arock)
 {
     struct cmd_item *ti;
     afs_int32 port = 0;
@@ -125,7 +120,7 @@ bc_DeleteHostCmd(as, arock)
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -151,8 +146,8 @@ bc_DeleteHostCmd(as, arock)
 
 	code = bc_SaveHosts();
 	if (code) {
-	    com_err(whoami, code, "Cannot save tape hosts");
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, code, "Cannot save tape hosts");
+	    afs_com_err(whoami, 0,
 		    "Changes are temporary - for this session only");
 	    ERROR(code);
 	}
@@ -175,17 +170,15 @@ bc_DeleteHostCmd(as, arock)
  *	ignored
  */
 
-afs_int32
-bc_ListHostsCmd(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+int
+bc_ListHostsCmd(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
     register struct bc_hostEntry *tentry;
 
     code = bc_UpdateHosts();
     if (code) {
-	com_err(whoami, code, "; Can't retrieve tape hosts");
+	afs_com_err(whoami, code, "; Can't retrieve tape hosts");
 	return (code);
     }
 
@@ -265,7 +258,7 @@ bc_ParseHosts()
 	sscanf(tbuffer, "%s %u", hostName, &port);
 	th = gethostbyname(hostName);
 	if (th == 0) {
-	    com_err(whoami, 0,
+	    afs_com_err(whoami, 0,
 		    "can't get host info for %s from nameserver or /etc/hosts.",
 		    hostName);
 	}
@@ -379,7 +372,7 @@ bc_UpdateHosts()
     }
 
     if (ctPtr->textVersion != -1) {
-	com_err(whoami, 0, "obsolete tapehosts - updating");
+	afs_com_err(whoami, 0, "obsolete tapehosts - updating");
 	bc_ClearHosts();
     }
 
@@ -397,7 +390,7 @@ bc_UpdateHosts()
 
     /* fetch the version number */
     code =
-	ubik_Call(BUDB_GetTextVersion, uhptr->uh_client, 0, ctPtr->textType,
+	ubik_BUDB_GetTextVersion(uhptr->uh_client, 0, ctPtr->textType,
 		  &ctPtr->textVersion);
     if (code)
 	ERROR(code);
