@@ -1,3 +1,4 @@
+
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
@@ -112,7 +113,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptserver.c,v 1.19 2003/11/29 22:08:15 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/ptserver.c,v 1.21.2.14 2008/12/22 19:23:32 shadow Exp $");
 
 #include <afs/stds.h>
 #ifdef	AFS_AIX32_ENV
@@ -120,6 +121,8 @@ RCSID
 #endif
 #include <sys/types.h>
 #include <stdio.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <WINNT/afsevent.h>
@@ -127,13 +130,7 @@ RCSID
 #include <netdb.h>
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_STRING_H
 #include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
@@ -146,6 +143,7 @@ RCSID
 #include "error_macros.h"
 #include "afs/audit.h"
 #include <afs/afsutil.h>
+#include <afs/com_err.h>
 
 
 /* make	all of these into a structure if you want */
@@ -163,14 +161,48 @@ extern int afsconf_CheckAuth();
 int pr_realmNameLen;
 char *pr_realmName;
 
+int debuglevel = 0;
+int restricted = 0;
+int rxMaxMTU = -1;
+int rxBind = 0;
+int rxkadDisableDotCheck = 0;
+
+#define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
+afs_uint32 SHostAddrs[ADDRSPERSITE];
+
 static struct afsconf_cell info;
 
+extern int prp_group_default;
+extern int prp_user_default;
+
 #include "AFS_component_version_number.c"
 
+int
+prp_access_mask(s)
+    char *s;
+{
+    int r;
+    if (*s >= '0' && *s <= '9') {
+	return strtol(s, NULL, 0);
+    }
+    r = 0;
+    while (*s) switch(*s++)
+    {
+    case 'S':	r |= PRP_STATUS_ANY; break;
+    case 's':	r |= PRP_STATUS_MEM; break;
+    case 'O':	r |= PRP_OWNED_ANY; break;
+    case 'M':	r |= PRP_MEMBER_ANY; break;
+    case 'm':	r |= PRP_MEMBER_MEM; break;
+    case 'A':	r |= PRP_ADD_ANY; break;
+    case 'a':	r |= PRP_ADD_MEM; break;
+    case 'r':	r |= PRP_REMOVE_MEM; break;
+    }
+    return r;
+}
+
 /* check whether caller is authorized to manage RX statistics */
 int
-pr_rxstat_userok(call)
-     struct rx_call *call;
+pr_rxstat_userok(struct rx_call *call)
 {
     return afsconf_SuperUser(prdir, call, NULL);
 }
@@ -192,6 +224,7 @@ main(int argc, char **argv)
     int kerberosKeys;		/* set if found some keys */
     int lwps = 3;
     char clones[MAXHOSTSPERCELL];
+    afs_uint32 host = htonl(INADDR_ANY);
 
     const char *pr_dbaseName;
     char *whoami = "ptserver";
@@ -214,6 +247,7 @@ main(int argc, char **argv)
     sigaction(SIGABRT, &nsa, NULL);
     sigaction(SIGSEGV, &nsa, NULL);
 #endif
+    osi_audit_init();
     osi_audit(PTS_StartEvent, 0, AUD_END);
 
     /* Initialize dirpaths */
@@ -248,6 +282,13 @@ main(int argc, char **argv)
 	if ((strncmp(arg, "-database", alen) == 0)
 	    || (strncmp(arg, "-db", alen) == 0)) {
 	    pr_dbaseName = argv[++a];	/* specify a database */
+	} else if (strcmp(argv[a], "-d") == 0) {
+	    if ((a + 1) >= argc) {
+		fprintf(stderr, "missing argument for -d\n"); 
+		return -1; 
+	    }
+	    debuglevel = atoi(argv[++a]);
+	    LogLevel = debuglevel;
 	} else if (strncmp(arg, "-p", alen) == 0) {
 	    lwps = atoi(argv[++a]);
 	    if (lwps > 16) {	/* maximum of 16 */
@@ -267,6 +308,19 @@ main(int argc, char **argv)
 	    depthsg = atoi(argv[++a]);	/* Max search depth for supergroups */
 	}
 #endif
+	else if (strncmp(arg, "-default_access", alen) == 0) {
+	    prp_user_default = prp_access_mask(argv[++a]);
+	    prp_group_default = prp_access_mask(argv[++a]);
+	}
+	else if (strncmp(arg, "-restricted", alen) == 0) {
+	    restricted = 1;
+	}
+	else if (strncmp(arg, "-rxbind", alen) == 0) {
+	    rxBind = 1;
+	}
+	else if (strncmp(arg, "-allow-dotted-principals", alen) == 0) {
+	    rxkadDisableDotCheck = 1;
+	}
 	else if (strncmp(arg, "-enable_peer_stats", alen) == 0) {
 	    rx_enablePeerRPCStats();
 	} else if (strncmp(arg, "-enable_process_stats", alen) == 0) {
@@ -281,31 +335,67 @@ main(int argc, char **argv)
 	    serverLogSyslogFacility = atoi(arg + 8);
 	}
 #endif
+	else if (strncmp(arg, "-auditlog", alen) == 0) {
+	    char *fileName = argv[++a];
+
+            osi_audit_file(fileName);
+            osi_audit(PTS_StartEvent, 0, AUD_END);
+	}
+	else if (!strncmp(arg, "-rxmaxmtu", alen)) {
+	    if ((a + 1) >= argc) {
+		fprintf(stderr, "missing argument for -rxmaxmtu\n");
+		PT_EXIT(1);
+	    }
+	    rxMaxMTU = atoi(argv[++a]);
+	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
+		 (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
+		printf("rxMaxMTU %d invalid; must be between %d-%d\n",
+			rxMaxMTU, RX_MIN_PACKET_SIZE,
+			RX_MAX_PACKET_DATA_SIZE);
+		PT_EXIT(1);
+	    }
+	} 
 	else if (*arg == '-') {
 	    /* hack in help flag support */
 
 #if defined(SUPERGROUPS)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: ptserver [-database <db path>] "
-		   "[-syslog[=FACILITY]] "
+		   "[-auditlog <log path>] "
+		   "[-syslog[=FACILITY]] [-d <debug level>] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-groupdepth <depth>] "
+		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-allow-dotted-principals] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-default_access default_user_access default_group_access] "
 		   "[-help]\n");
 #else /* AFS_NT40_ENV */
 	    printf("Usage: ptserver [-database <db path>] "
-		   "[-p <number of processes>] [-rebuild] "
+		   "[-auditlog <log path>] [-d <debug level>] "
+		   "[-p <number of processes>] [-rebuild] [-rxbind] "
+		   "[-allow-dotted-principals] "
+		   "[-default_access default_user_access default_group_access] "
+		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
 		   "[-groupdepth <depth>] " "[-help]\n");
 #endif
 #else
 #ifndef AFS_NT40_ENV
 	    printf("Usage: ptserver [-database <db path>] "
+		   "[-auditlog <log path>] [-d <debug level>] "
 		   "[-syslog[=FACILITY]] "
 		   "[-p <number of processes>] [-rebuild] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-default_access default_user_access default_group_access] "
+		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-allow-dotted-principals] "
 		   "[-help]\n");
 #else /* AFS_NT40_ENV */
 	    printf("Usage: ptserver [-database <db path>] "
+		   "[-auditlog <log path>] [-d <debug level>] "
+		   "[-default_access default_user_access default_group_access] "
+		   "[-restricted] [-rxmaxmtu <bytes>] [-rxbind] "
+		   "[-allow-dotted-principals] "
 		   "[-p <number of processes>] [-rebuild] " "[-help]\n");
 #endif
 #endif
@@ -356,7 +446,7 @@ main(int argc, char **argv)
     code =
 	afsconf_GetExtendedCellInfo(prdir, NULL, "afsprot", &info, &clones);
     if (code) {
-	com_err(whoami, code, "Couldn't get server list");
+	afs_com_err(whoami, code, "Couldn't get server list");
 	PT_EXIT(2);
     }
     pr_realmName = info.name;
@@ -366,7 +456,7 @@ main(int argc, char **argv)
     /* get keys */
     code = afsconf_GetKey(prdir, 999, &tkey);
     if (code) {
-	com_err(whoami, code,
+	afs_com_err(whoami, code,
 		"couldn't get bcrypt keys from key file, ignoring.");
     }
 #endif
@@ -398,11 +488,33 @@ main(int argc, char **argv)
      * and the header are in separate Ubik buffers then 120 buffers may be
      * required. */
     ubik_nBuffers = 120 + /*fudge */ 40;
+
+    if (rxBind) {
+	afs_int32 ccode;
+#ifndef AFS_NT40_ENV
+	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
+	    AFSDIR_SERVER_NETINFO_FILEPATH) {
+	    char reason[1024];
+	    ccode = parseNetFiles(SHostAddrs, NULL, NULL,
+					   ADDRSPERSITE, reason,
+					   AFSDIR_SERVER_NETINFO_FILEPATH,
+					   AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	} else 
+#endif
+	{
+	    ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
+	}
+	if (ccode == 1) {
+	    host = SHostAddrs[0];
+	    rx_InitHost(host, htons(AFSCONF_PROTPORT));
+	}
+    }
+
     code =
-	ubik_ServerInitByInfo(myHost, htons(AFSCONF_PROTPORT), &info, &clones,
+	ubik_ServerInitByInfo(myHost, htons(AFSCONF_PROTPORT), &info, clones,
 			      pr_dbaseName, &dbase);
     if (code) {
-	com_err(whoami, code, "Ubik init failed");
+	afs_com_err(whoami, code, "Ubik init failed");
 	PT_EXIT(2);
     }
 #if defined(SUPERGROUPS)
@@ -419,8 +531,12 @@ main(int argc, char **argv)
     /* Disable jumbograms */
     rx_SetNoJumbo();
 
+    if (rxMaxMTU != -1) {
+	rx_SetMaxMTU(rxMaxMTU);
+    }
+
     tservice =
-	rx_NewService(0, PRSRV, "Protection Server", sc, 3,
+	rx_NewServiceHost(host, 0, PRSRV, "Protection Server", sc, 3,
 		      PR_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	fprintf(stderr, "ptserver: Could not create new rx service.\n");
@@ -428,9 +544,14 @@ main(int argc, char **argv)
     }
     rx_SetMinProcs(tservice, 2);
     rx_SetMaxProcs(tservice, lwps);
+    if (rxkadDisableDotCheck) {
+        rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
+                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
+                                    NULL);
+    }
 
     tservice =
-	rx_NewService(0, RX_STATS_SERVICE_ID, "rpcstats", sc, 3,
+	rx_NewServiceHost(host, 0, RX_STATS_SERVICE_ID, "rpcstats", sc, 3,
 		      RXSTATS_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
 	fprintf(stderr, "ptserver: Could not create new rx service.\n");
