@@ -15,7 +15,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/ubikclient.c,v 1.8 2003/07/15 23:17:05 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/ubikclient.c,v 1.8.2.3 2006/07/31 17:07:51 shadow Exp $");
 
 #if defined(UKERNEL)
 #include "afs/sysincludes.h"
@@ -46,7 +46,6 @@ RCSID
 #endif /* defined(UKERNEL) */
 
 
-afs_int32 ubik_CallIter();
 short ubik_initializationState;	/* initial state is zero */
 
 
@@ -72,14 +71,17 @@ ubik_ParseClientList(int argc, char **argv, afs_int32 * aothers)
 	    if (*tp == '-')
 		break;		/* done */
 	    /* otherwise this is a new host name */
-	    LOCK_GLOBAL_MUTEX th = gethostbyname(tp);
+	    LOCK_GLOBAL_MUTEX;
+	    th = gethostbyname(tp);
 	    if (!th) {
-		UNLOCK_GLOBAL_MUTEX return UBADHOST;
+		UNLOCK_GLOBAL_MUTEX;
+		return UBADHOST;
 	    }
 	    memmove((void *)&temp, (const void *)th->h_addr,
 		    sizeof(afs_int32));
-	    UNLOCK_GLOBAL_MUTEX if (counter++ >= MAXSERVERS)
-		  return UNHOSTS;
+	    UNLOCK_GLOBAL_MUTEX;
+	    if (counter++ >= MAXSERVERS)
+		return UNHOSTS;
 	    *aothers++ = temp;
 	} else {
 	    /* haven't seen a -server yet */
@@ -205,11 +207,11 @@ ubik_ClientInit(register struct rx_connection **serverconns,
     initialize_U_error_table();
 
     if (*aclient) {		/* the application is doing a re-initialization */
-	LOCK_UBIK_CLIENT((*aclient))
-	    /* this is an important defensive check */
-	    if (!((*aclient)->initializationState)) {
-	    UNLOCK_UBIK_CLIENT((*aclient))
-		return UREINITIALIZE;
+	LOCK_UBIK_CLIENT((*aclient));
+	/* this is an important defensive check */
+	if (!((*aclient)->initializationState)) {
+	    UNLOCK_UBIK_CLIENT((*aclient));
+	    return UREINITIALIZE;
 	}
 
 	/* release all existing connections */
@@ -223,9 +225,9 @@ ubik_ClientInit(register struct rx_connection **serverconns,
 	    rx_DestroyConnection(rxConn);
 #endif
 	}
-	UNLOCK_UBIK_CLIENT((*aclient))
+	UNLOCK_UBIK_CLIENT((*aclient));
 #ifdef AFS_PTHREAD_ENV
-	    if (pthread_mutex_destroy(&((*aclient)->cm)))
+	if (pthread_mutex_destroy(&((*aclient)->cm)))
 	    return UMUTEXDESTROY;
 #endif
     } else {
@@ -304,8 +306,8 @@ ubik_ClientDestroy(struct ubik_client * aclient)
  *     error.
  */
 
-static struct rx_connection *
-RefreshConn(struct rx_connection *tc)
+struct rx_connection *
+ubik_RefreshConn(struct rx_connection *tc)
 {
     afs_uint32 host;
     u_short port;
@@ -335,8 +337,8 @@ pthread_once_t ubik_client_once = PTHREAD_ONCE_INIT;
 pthread_mutex_t ubik_client_mutex;
 #define LOCK_UCLNT_CACHE \
     assert(pthread_once(&ubik_client_once, ubik_client_init_mutex) == 0 && \
-	   pthread_mutex_lock(&ubik_client_mutex)==0);
-#define UNLOCK_UCLNT_CACHE assert(pthread_mutex_unlock(&ubik_client_mutex)==0);
+	   pthread_mutex_lock(&ubik_client_mutex)==0)
+#define UNLOCK_UCLNT_CACHE assert(pthread_mutex_unlock(&ubik_client_mutex)==0)
 
 void
 ubik_client_init_mutex()
@@ -397,18 +399,19 @@ ubik_Call(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10,
     rcode = UNOSERVERS;
     chaseCount = inlist = needsync = 0;
 
-    LOCK_UCLNT_CACHE for (j = 0; ((j < SYNCCOUNT) && calls_needsync[j]); j++) {
+    LOCK_UCLNT_CACHE;
+    for (j = 0; ((j < SYNCCOUNT) && calls_needsync[j]); j++) {
 	if (calls_needsync[j] == (int *)aproc) {
 	    inlist = needsync = 1;
 	    break;
 	}
     }
-    UNLOCK_UCLNT_CACHE
-	/* 
-	 * First  pass, we try all servers that are up.
-	 * Second pass, we try all servers.
-	 */
-	for (pass = 0; pass < 2; pass++) {	/*p */
+    UNLOCK_UCLNT_CACHE;
+    /* 
+     * First  pass, we try all servers that are up.
+     * Second pass, we try all servers.
+     */
+    for (pass = 0; pass < 2; pass++) {	/*p */
 	/* For each entry in our servers list */
 	for (count = 0;; count++) {	/*s */
 
@@ -425,7 +428,7 @@ ubik_Call(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10,
 		     */
 		    tc = aclient->conns[count];
 		    if (tc && rx_ConnError(tc)) {
-			aclient->conns[count] = tc = RefreshConn(tc);
+			aclient->conns[count] = tc = ubik_RefreshConn(tc);
 		    }
 		    if (!tc)
 			break;
@@ -460,7 +463,7 @@ ubik_Call(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10,
 	    /*needsync */
 	    tc = aclient->conns[count];
 	    if (tc && rx_ConnError(tc)) {
-		aclient->conns[count] = tc = RefreshConn(tc);
+		aclient->conns[count] = tc = ubik_RefreshConn(tc);
 	    }
 	    if (!tc)
 		break;
@@ -494,10 +497,11 @@ ubik_Call(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10,
   done:
     if (needsync) {
 	if (!inlist) {		/* Remember proc call that needs sync site */
-	    LOCK_UCLNT_CACHE calls_needsync[synccount % SYNCCOUNT] =
-		(int *)aproc;
+	    LOCK_UCLNT_CACHE;
+	    calls_needsync[synccount % SYNCCOUNT] = (int *)aproc;
 	    synccount++;
-	    UNLOCK_UCLNT_CACHE inlist = 1;
+	    UNLOCK_UCLNT_CACHE;
+	    inlist = 1;
 	}
 	if (!rcode) {		/* Remember the sync site - cmd successful */
 	    rxp = rx_PeerOf(aclient->conns[count]);
@@ -533,7 +537,7 @@ try_GetSyncSite(register struct ubik_client *aclient, afs_int32 apos)
     /* get this conn */
     tc = aclient->conns[apos];
     if (tc && rx_ConnError(tc)) {
-	aclient->conns[apos] = (tc = RefreshConn(tc));
+	aclient->conns[apos] = (tc = ubik_RefreshConn(tc));
     }
     if (!tc) {
 	return -1;
@@ -605,7 +609,7 @@ CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7, p8, p9,
     short origLevel;
 
     if (needlock) {
-	LOCK_UBIK_CLIENT(aclient)
+	LOCK_UBIK_CLIENT(aclient);
     }
     origLevel = aclient->initializationState;
 
@@ -616,13 +620,13 @@ CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7, p8, p9,
 	tc = aclient->conns[*apos];
 	if (!tc) {
 	    if (needlock) {
-		UNLOCK_UBIK_CLIENT(aclient)
+		UNLOCK_UBIK_CLIENT(aclient);
 	    }
 	    return UNOSERVERS;
 	}
 
 	if (rx_ConnError(tc)) {
-	    tc = RefreshConn(tc);
+	    tc = ubik_RefreshConn(tc);
 	    aclient->conns[*apos] = tc;
 	}
 
@@ -634,7 +638,7 @@ CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7, p8, p9,
     }
     if (*apos >= MAXSERVERS) {
 	if (needlock) {
-	    UNLOCK_UBIK_CLIENT(aclient)
+	    UNLOCK_UBIK_CLIENT(aclient);
 	}
 	return UNOSERVERS;
     }
@@ -644,7 +648,7 @@ CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7, p8, p9,
 		  p14, p15, p16);
     if (aclient->initializationState != origLevel) {
 	if (needlock) {
-	    UNLOCK_UBIK_CLIENT(aclient)
+	    UNLOCK_UBIK_CLIENT(aclient);
 	}
 	return code;		/* somebody did a ubik_ClientInit */
     }
@@ -659,7 +663,7 @@ CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7, p8, p9,
 
     (*apos)++;
     if (needlock) {
-	UNLOCK_UBIK_CLIENT(aclient)
+	UNLOCK_UBIK_CLIENT(aclient);
     }
     return code;
 }
@@ -698,7 +702,7 @@ ubik_Call_New(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10,
     int stepBack;
     short origLevel;
 
-    LOCK_UBIK_CLIENT(aclient)
+    LOCK_UBIK_CLIENT(aclient);
   restart:
     rcode = UNOSERVERS;
     origLevel = aclient->initializationState;
@@ -732,41 +736,24 @@ ubik_Call_New(aproc, aclient, aflags, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10,
 		    }
 		}
 	    } else if ((code >= 0) && (code != UNOQUORUM)) {
-		UNLOCK_UBIK_CLIENT(aclient)
-		    return code;	/* success or global error condition */
+		UNLOCK_UBIK_CLIENT(aclient);
+		return code;	/* success or global error condition */
 	    }
 	}
     }
-    UNLOCK_UBIK_CLIENT(aclient)
-	return rcode;
+    UNLOCK_UBIK_CLIENT(aclient);
+    return rcode;
 }
 
 /* 
  * This is part of an iterator.  It doesn't handle finding sync sites
  */
 afs_int32
-ubik_CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7, p8,
-	      p9, p10, p11, p12, p13, p14, p15, p16)
-     int (*aproc) ();
-     register struct ubik_client *aclient;
-     afs_int32 aflags;
-     int *apos;
-     long p1;
-     long p2;
-     long p3;
-     long p4;
-     long p5;
-     long p6;
-     long p7;
-     long p8;
-     long p9;
-     long p10;
-     long p11;
-     long p12;
-     long p13;
-     long p14;
-     long p15;
-     long p16;
+ubik_CallIter(int (*aproc) (), struct ubik_client *aclient,
+			       afs_int32 aflags, int *apos, long p1, long p2,
+			       long p3, long p4, long p5, long p6, long p7,
+			       long p8, long p9, long p10, long p11, long p12,
+			       long p13, long p14, long p15, long p16)
 {
     return CallIter(aproc, aclient, aflags, apos, p1, p2, p3, p4, p5, p6, p7,
 		    p8, p9, p10, p11, p12, p13, p14, p15, p16, NEED_LOCK);
